% ApplyPCA Apply PCA to the hyperspectral image to find the most
% principle components of the hyper images
%
%   IM = APPLYPCA(DATA, N)
%
% INPUT
%      DATA: input dataset of hyperspectral image
%      N   : number of principle components needed to find out
% OUTPUT
%      IM: Image of the principle components
%
% DESCRIPTION
% Extract the most N principle components of hyper images
% In case N = 1: represents data as a gray scale image
% In case N = 3: represents data as a RGB image
%
%
% Copyright: Dinh Viet Cuong, d.v.cuong@tudelft.nl
% ICT group, Elect. Eng., Maths and Comp. Sc, 
% Delft University of Technology
% P.O. Box 5046, 2600 GA Delft, The Netherlands

% $Id: applyPCA.m,v 1.1 2009/01/16 15:03:50 dcuong Exp $

function im = applyPCA(a, N)
	if nargin == 0
		error('error!, input dataset name');
	elseif ~isa(a,'dataset')
		error('error!, input dataset name');	
	else
		image1 = [];
		% Convert dataset from feature type to object type
		image1 = data2im(a);
		a = im2obj(image1);
		
		% Apply PCA to hyper image
		w = pca(a);
		
		% Convert mapping to dataset
		[k,c] = size(w);
		a = dataset(w.data.rot',w.labels(1:c,:));
		a = setfeatsize(a,w.size_in);		
		
		switch N
			case 1
				a1 = a(1,:);
				%print into file
				%figure; show(a1);
				%print -f1 -djpg component1.jpg
				I1 = data2im(a1);
				im = mat2gray(I1);
			case 3
				a1 = a(1,:); I1 = data2im(a1);
				a2 = a(2,:); I2 = data2im(a2);
				a3 = a(3,:); I3 = data2im(a3);
				
				im1 = mat2gray(I1); im(:,:,1) = im1;
				im2 = mat2gray(I2); im(:,:,2) = im2;
				im3 = mat2gray(I3); im(:,:,3) = im3;
				
				% show image
				%imshow(im);		
		end
	end

end
