%CCA_APPLY Canonical Correlation Analysis (applying)
%  
%  [T,U] = cca_apply(X,Y,W,Q)
%  [T,U] = cca_apply(X,Y,W,Q,Options)
%
% INPUT
%   X       [N -by- d_X] input data matrix, N samples, d_X variables; if X==[] then T==[] 
%   Y       [N -by- d_Y] output data matrix, N samples, d_Y variables; if Y==[] then U==[] 
%   W       [d_X -by- n] X weights  (X transformation matrix)
%   Q       [d_Y -by- n] Y weights  (Y transformation matrix)
%   Options structure returned by cca_train; if not supplied then will be no preprocessing performed
%
% OUTPUT
%   T [N -by- n]  X scores (transformed (preprocessed) X)
%   U [N -by- n]  Y scores (transformed (preprocessed) Y)
%
% DESCRIPTION
% Applys precalculated CCA transformation to the new data.
%
% SEE ALSO
% CCA_TRAIN

% Copyright: S. Verzakov, serguei@ph.tn.tudelft.nl
% Faculty of Applied Sciences, Delft University of Technology
% P.O. Box 5046, 2600 GA Delft, The Netherlands

% $Id: cca_apply.m,v 1.5 2005/02/07 10:03:21 pavel Exp $

function [T,U] = cca_apply(X,Y,W,Q,Options)

T = [];
U = [];

if nargin < 4 | nargin > 5
  error('Invalid number of parameters');
end

if nargin < 5
  Options  = [];
end

if ~isempty(X)
  if isempty(W)
    error('W cannot be empty if X is not empty');
  end
   
  if size(X,2) ~= size(W,1)
    error('size(X,2) ~= size(W,1)');
  end
end

if ~isempty(Y)
  if isempty(Q)
    error('Q cannot be empty if Y is not empty');
  end
   
  if size(Y,2) ~= size(Q,1)
    error('size(Y,2) ~= size(Q,1)');
  end
end


DefaultOptions.X_centering = [];
DefaultOptions.Y_centering = [];
DefaultOptions.X_scaling = [];
DefaultOptions.Y_scaling = [];

Options = updstruct(DefaultOptions, Options);

[N_X, d_X] = size(X);
[N_Y, d_Y] = size(Y);

if N_X ~= N_Y
	error('size(X,1) must be equal to size(Y,1)');
else
	N = N_X;
end

if ~isempty(X)
  X = prepro(X, Options.X_centering, Options.X_scaling);
  T = X*W;
end  
  
if ~isempty(Y)
  Y = prepro(Y, Options.Y_centering, Options.Y_scaling);
  U = Y*Q;
end  

return;
