%CCAM Canonical Correlation Analysis mapping
%
%  W = CCAM
%  W = CCAM*A
%
% DESCRIPTION
% CCAM routine provides a prtools mapping for Cannonical Correlation
% Analysis, based on CCA_TRAIN and CCA_APPLY functions.
%
% SEE ALSO
% CCA_TRAIN, CCA_APPLY

% $Id: ccam.m,v 1.7 2005/05/10 11:02:56 serguei Exp $

function [w,info]=ccam(par1,par2,par3)

% No dataset given: return untrained mapping.
if (nargin < 1) | (isempty(par1))
  if nargin < 2
    par2 = 0;
  end  
  if nargin < 3
    par3 = 0;
  end  
  data = {par2, par3};
  w = mapping(mfilename,'untrained',data);
  w = setname(w,'Cannonical Correlation Analysis Mapping');
  return
end

isdataset(par1);           % Assert that A is a dataset.

% training
if nargin < 2 | ~isa(par2,'mapping') 
  % a*w when w is untrained or ccam(a,,) 
  if nargin < 2
    par2 = 0;
  end  
  if nargin < 3
    par3 = 0;
  end  
  flag_t = par2; 
  flag_u = par3;
  
  [m,k] = getsize(par1);
  
  if strcmp(par1.labtype,'crisp')
    y=gettargets(setlabtype(par1,'soft'));
  else
    y=gettargets(par1);
  end
  
  % options
  Options.X_centering=[];
  Options.Y_centering=[];
  Options.X_scaling=[];
  Options.Y_scaling=[];
  
  if flag_t
    data.info = 't_train''*t_train=m_train; ';
    Options.M_T=m;
  else
    data.info = 't_train''*t_train=m_train-1; ';
    Options.M_T=m-1;
  end
  
  if flag_u
    data.info = [data.info 'u_train''*u_train=m_train;'];
    Options.M_U=m;
  else
    data.info = [data.info 'u_train''*u_train=m_train-1;'];
    Options.M_U=m-1;  
  end
  
  [cr,XRes,YRes,Options]=cca_train(+par1,y,Options);
  
  data.W=XRes.W;
  data.Q=YRes.Q;
  data.Options=Options;
  
  % Save all useful data
  w = mapping(mfilename,'trained',data,[],size(XRes.W,1),size(XRes.W,2));
  w = setname(w,'Cannonical Correlation Analysis Mapping');
  
  if nargout > 1
    info.cr   = cr;
    info.XRes = XRes;
    info.YRes = YRes;
  end
  
  % applying
else 
  data = getdata(par2); % Unpack the mapping.
  
  if strcmp(par1.labtype,'crisp')
    y=gettargets(setlabtype(par1,'soft'));
  else
    y=gettargets(par1);
  end
  
  [T,U] = cca_apply(+par1,y,data.W,data.Q,data.Options);
  
  w = setdat(par1,T,par2);
  %w = setlabtype(w,'targets');
  %w = settargets(w,U);
  
  if nargout > 1
    info = U;
  end
end

return


