%CHEMIMAGE Create an image rendering the area under the part of spectra
%
%  IMOUT = CHEMIMAGE(IM,BANDS,VALUES)
%
% INPUT
%   IM      spectral image (dip_image)
%   BANDS   vector with two bands/wavelengths/wavenumbers indices
%
% OUTPUT
%  IMOUT   gray-valued dip image (each pixels corresponds to area under the selected part of spectra); 
%
% DESCRIPTION
% The routine produces an image rendering the area under the part of
% spectra, defined by BANDS. 
%
% SEE ALSO
% SHOWSI

% Copyright: Pavel Paclik, pavel@ph.tn.tudelft.nl
% Faculty of Applied Physics, Delft University of Technology
% P.O. Box 5046, 2600 GA Delft, The Netherlands

function imout=chemimage(im,bands)

	if ~isa(im,'dip_image')
		error('dip_image required');
	end
	
	% lets fix the ordering by sorting:
	[bands,ind]=sort(bands);

	% area under the spectra
	im=double(im(:,:,bands(1):bands(2)));
	area=sum(im,3);
	
	% area under the baselines
	
	% we compute the area of trapezoid:
        %      /| v2 	
        %  v1 / |       area=(band2-band1)*mean(v1,v2)
        %     | |
	%     ---
	% band1 band2

	t=zeros(size(im,1),size(im,2),2);
	t(:,:,1)=im(:,:,1);
	t(:,:,2)=im(:,:,2);	
	t=mean(t,3); % mean of values
	
	area=area - (t .* (bands(2)-bands(1)));
	imout=dip_image( area );
	
	return
