%CORRC Correlation classifier
% 
%  W = CORRC(A)
% 
% INPUT  
%  A    Training dataset
%
% OUTPUT
%  W    Correlation classifier mapping
%
% DESCRIPTION
% By correlation classifier the new sample is prescribed to the class with 
% whose mean it is the most co-directed (has the smallest angle)
%
% SEE ALSO
% DATASETS, MAPPINGS

% Copyright: P. Paclik, pavel@ph.tn.tudelft.nl
% Faculty of Applied Physics, Delft University of Technology
% P.O. Box 5046, 2600 GA Delft, The Netherlands

% $Id: corrc.m,v 1.3 2003/12/09 13:19:59 pavel Exp $

function w = corrc(a,arg2)

	% No arguments given: return untrained mapping.

	if (nargin < 1) | (isempty(a))
		w = mapping(mfilename);
		w = setname(w,'Correlation classifier');
		return
	end

	if (~exist('arg2','var'))     % Second argument is not a mapping: training.

		isdataset(a);           % Assert that A is a dataset.

		[m,k,c] = getsize(a); M = classsizes(a);

		% class means
		pars.means=meancov(a);
		
		% Save all useful data.

		w = mapping(mfilename,'trained',pars,getlablist(a),k,c);
		w = setname(w,'Correlation classifier');

	else                      % Second argument is a a mapping: testing.

		w = arg2;
		pars = getdata(w);		 	% Unpack the mapping.
		[m,k] = getsize(a); 

		c = size( pars.means, 1 );
		t=+a;

		% computing sqrt(x'x)
		X = sqrt(sum(t.*t,2));

		% class means
		means=+pars.means;
		M = sqrt(sum(means.*means,2));

		out= (t * means')./ (X*M');
		w = setdat(a,out,w);

	end

	return



