%DASAM Spectral Angle Mapper (using acos)
%
%  D=DASAM(DATA)
%  D=DASAM(DATA,PROTO)
%
% INPUT 
%  DATA    testing dataset
%  PROTO   training dataset (representation set)
% 
% OUTPUT 
%  D       dataset with a distance (angles) matrix
%
% DESCRIPTION   
% Variables in the output dataset D are angles (in the [0; pi) region) 
% between DATA samples and PROTO samples. The number of samples in D is equal to the 
% one of  DATA. D.OBJSIZE is the same as DATA.OBJSIZE. If PROTO is not supplied then
% PROTO==DATA.
%
% SEE ALSO
% DSAM

% Copyright: Pavel Paclik, pavel@ph.tn.tudelft.nl
% Faculty of Applied Physics, Delft University of Technology
% P.O. Box 5046, 2600 GA Delft, The Netherlands

% $Id: dasam.m,v 1.12 2005/02/10 15:24:07 pavel Exp $

function d=dasam(data,proto)

      if ~exist('proto','var'), proto=data; end

      if size(data,2)~=size(proto,2)
	      error('both datasets must have the same feature sizes');
      end

      sc=size(data,1); % training sample size
      pc=size(proto,1); % test sample size

      labdata=getlab(data);
      labproto=getlab(proto);
      objsize=data.objsize;
      data2=data;
      proto2=proto;
      data=+data;
      proto=+proto;

      ndata=sqrt(sum(data.*data,2));
      nproto=sqrt(sum(proto.*proto,2));

      d = acos((data*proto')./(ndata*nproto'));

      d=setdata(data2,real(d),getlab(proto2));
      
      return
      
