%DATA2DIP Convert image in a dataset into a dip image
%
%  IM=DATA2DIP(A)
%  IM=DATA2DIP(A,TYPE)
%
% INPUT
%  A       dataset
%  TYPE    data type (default: 'double'), see dip_image help 
%          for all possible types
%
% OUTPUT
%  IM      dip_image object
%
% DESCRIPTION
% Conversion of a hyperspectral dataset into the dip image object.
% The dataset object must have the DATASIZE parameter set.
%
% SEE ALSO
% DIP2DATA

% Copyright: Pavel Paclik, P.Paclik@ewi.tudelft.nl

% $Id: data2dip.m,v 1.11 2005/11/02 14:57:58 pavel Exp $

function im=data2dip(a,type)
	
	if ~isdipinstalled
		error('DIPimage not installed');
	end
	
	isdataset(a);
	
	if ~exist('type','var')
		type='double';
	end
	
	objsize=a.objsize;
	if length(objsize)==2
		im=dip_image(...
		    reshape(+a,[objsize(1),objsize(2),size(a,2)]),...
		    type );
	elseif isvalidprop(a,'pos')
	    % irregular object
	    pos=getprop(a,'pos');
	    if size(pos,2)~=2, error('2D pos matrix expected'); end

	    bb=max(pos)-min(pos)+1;
    
	    % shift origin to leftmost pixel of the bounding box
	    pos=pos-repmat(min(pos),size(pos,1),1)+1;
	    
	    pixind=sub2ind(bb,pos(:,1),pos(:,2));

	    im=zeros([bb,size(a,2)]);
	    im=reshape(im,prod(bb),size(a,2));

	    im(pixind,:)=+a;
	    
	    im=reshape(im,[bb,size(a,2)]);

	    im=dip_image(im);

	else
	    error('dataset does not contain image data');
	end
	
	return
	
