%DCITYBLOCK City-block (Manhattan) distance
%
%  D=DCITYBLOCK(DATA)
%  D=DCITYBLOCK(DATA,PROTO)
%
% INPUT 
%  DATA    testing dataset
%  PROTO   training dataset (representation set)
% 
% OUTPUT 
%  D       dataset with a distance matrix
%
% DESCRIPTION   
% Variables in the output dataset D are city-block distances between DATA
% samples and PROTO samples. The number of samples in D is equal to the one
% of DATA. D.OBJSIZE is the same as DATA.OBJSIZE. If PROTO is not supplied
% then PROTO==DATA.

% Copyright: Pavel Paclik, pavel@ph.tn.tudelft.nl
% Faculty of Applied Physics, Delft University of Technology
% P.O. Box 5046, 2600 GA Delft, The Netherlands

% $Id: dcityblock.m,v 1.3 2005/02/10 15:24:07 pavel Exp $

function d=dcityblock(data,proto)

if ~exist('proto','var'), proto=data; end

if size(data,2)~=size(proto,2)
  error('both datasets must have the same feature sizes');
end
sc=size(data,1); % training sample size
pc=size(proto,1); % test sample size

d=zeros(sc,pc);

labdata=getlab(data);
labproto=getlab(proto);
objsize=data.objsize;
data2=+data;
proto2=+proto;

% for computational reasons, we put the lopp over prototypes
% into a mex file dist:
if exist('distfunc')==3
    d=distfunc(data2,proto2);
else
    for i=1:pc
	t=repmat(proto2(i,:),sc,1);
	d(:,i)=sum(abs(data2-t),2);
    end
end

d=setdat(data,d);
d=setfeatlab(d,labproto);

return
  

