% DERFILTER Gaussian derivative filter
%
%     OUT=DERFILTER(DATA,SIGMA)
%
% INPUT
%   DATA  - dataset or data matrix
%   SIGMA - smoothing parameter
%
% OUTPUT
%   OUT   - output dataset or data matrix
%
% DESCRIPTION
% DERFILTER computes smoothed 1st Gaussian derivative of signals (spectra)
% in DATA using standard Matlab convolution tools. 

% $Id: derfilter.m,v 1.4 2005/02/10 15:24:07 pavel Exp $

function out=derfilter(data,sigma)
    
    
    if ~exist('sigma','var'), sigma=2.0; end
    % we use window -3 simga : +3 simga
    x = floor(-3*sigma):ceil(3*sigma);
    g = exp(-0.5*x.^2/sigma^2); 
%    g = g/sum(g);
    dg = -x.*g/sigma^2;
    kernel = dg;
    
    sc=size(data,1); % sample count
    fc=size(data,2);
    kc=length(kernel)+10;

    if isdataset(data)
	data2=+data; % internal raw data
    else
	data2=data;
    end
    
    % output
    out=zeros(size(data));
    
    % extend the data by mirroring the tails
    data2=[ data2(:,kc:-1:1) data2 data2(:,end:-1:fc-kc+1) ];
    
    outc=fc+2*kc+length(kernel)-1;  % size of the convolution output
    % indices of a part corresponging to the original signal
    ind = kc+ceil(length(kernel)/2):outc-kc-floor(length(kernel)/2);

    % convolve the signals
    for i=1:sc
	t=conv(data2(i,:),kernel);
	out(i,:)=t(ind);	
    end
	
    if isdataset(data)
	out=setdat(data,out);
    end