%DIP2DATA Convert a dip image into a dataset
%
%  D = DIP2DATA(DIP)
%
% INPUT
%  DIP   dip image
%
% OUTPUT
%  D     generated dataset
%
% DESCRIPTION
% Conversion of a dip image object into the hyperspectral dataset.
% The original image size is preserved in OBJSIZE parameter.
%
% SEE ALSO
% DATA2DIP

% Copyright: Pavel Paclik, pavel@ph.tn.tudelft.nl
% Faculty of Applied Physics, Delft University of Technology
% P.O. Box 5046, 2600 GA Delft, The Netherlands

% $Id: dip2data.m,v 1.7 2005/11/02 14:29:04 pavel Exp $

function d=dip2data(dip)

	
	if isa(dip,'dip_image')
	
		objsize=size(dip);
		if length(objsize)==2, objsize=[objsize 1]; end
		d=dataset(reshape(double(dip),objsize(1)*objsize(2),objsize(3)));
		d.objsize=[objsize(2) objsize(1)];
		
		% generate object identifiers
		d.ident=[1:prod(d.objsize)]';
	else
		error('dip_image required');
	end
		
	return
