%DKL Kullback-Leibler divergence
%
%     D=DKL(DATA,PROTO)
%
% INPUT
%   DATA   a dataset
%   PROTO  the dataset with prototypes (representation set)
% OUTPUT
%   D      dataset with a distance matrix
%
% DESCRIPTION
% The Kullback-Leibler divergence (pp. 151, Pattern Recognition, Theodoridis)
%

% $Id: dkl.m,v 1.3 2005/03/02 11:58:56 pavel Exp $

function d=dkl(data,proto)

fprintf(' dkl:');

if size(data,2)~=size(proto,2)
  error('both datasets must have the same feature sizes');
end
sc=size(data,1); % training sample size
pc=size(proto,1); % test sample size
d=zeros(sc,pc);

labdata=getlab(data);
labproto=getlab(proto);
data2=data; data=+data;
proto2=proto; proto=+proto;

% correct for zeros
proto((+proto==0))=0.00001;
data((+data==0))=0.00001;

step=round(pc/10);
for i=1:pc
  t=repmat(proto(i,:),sc,1);
  t=sum(log( t./data ).*data ,2) + sum(log( data./t ).*t ,2);
  d(:,i)=t;
  if mod(i,step)==0, fprintf('.'); end
end

d=setdat(data2,d);
d=setfeatlab(d,labproto);

return
  
