%DKL Kullback-Leibler divergence
%
%     D=DKL_TRUE(DATA,PROTO,DELTA)
%
% INPUT
%   DATA   a dataset
%   PROTO  the dataset with prototypes (representation set)
%   DELTA  regularization parameter (by default = 0)
% OUTPUT
%   D      dataset with a distance matrix
%
% DESCRIPTION
% The Kullback-Leibler divergence (pp. 151, Pattern Recognition,
% Theodoridis, p. 176 in 2nd edition)
%

% $Id: dkl_true.m,v 1.2 2005/04/08 13:46:17 serguei Exp $

function d=dkl_true(data,proto,delta)

fprintf(' dkl_true:');

if nargin < 3
  delta = 0;
end

if delta < 0
  error('regularization parameter delta is negative');
end

nb = size(proto,2);
if size(data,2)~=nb
  error('both datasets must have the same feature sizes');
end

ns=size(data,1);  % the number of sampes
np=size(proto,1); % the number of prototypes

labdata=getlab(data);
labproto=getlab(proto);


data2=data; data=+data;
proto2=proto; proto=+proto;


% normalization: we consider a histogram being a set of probability
% estimates
data=data./repmat(sum(data,2),[1,size(data,2)]);
proto=proto./repmat(sum(proto,2),[1,size(proto,2)]);

if delta > 0
  proto = (1-delta)*proto + delta/nb;
  data = (1-delta)*data + delta/nb;
  if ~isfinite(log(delta/nb))
    prwarning(1,'regularization parameter too small to prevent infinite distances');
  end
end

d=zeros(ns,np);
if np <= ns
  zind_data = find(data == 0);
  for j=1:np
    product = repmat(proto(j,:),[ns,1]);
    zind = zind_data(product(zind_data)==0);

    w = warning;
    warning('off');
    product = (data - product).*log(data./product);
    warning(w);  
    product(zind) = 0; 
    d(:,j) = sum(product,2);
  end

else  
  zind_proto = find(proto == 0);
  for i=1:ns
    product = repmat(data(i,:),[np,1]);
    zind = zind_proto(product(zind_proto)==0);

    w = warning;
    warning('off');
    product = (product - proto).*log(product./proto);
    warning(w);  
    product(zind) = 0; 
    d(i,:) = sum(product,2)';
  end
end

d=setdat(data2,d);
d=setfeatlab(d,labproto);

return
  
