%DKOLMOGOROV Kolmogorov distance
%
%  D=DKOLMOGOROV(DATA)
%  D=DKOLMOGOROV(DATA,PROTO)
%
% INPUT
%  DATA  testing dataset
%  PROTO training dataset (representation set)
% 
% OUTPUT
%  D     dataset with a distance matrix
%
% DESCRIPTION
% In these routine each spectrum is treated as a density of some distribution.
% "Cumulative distribution function" for each spectrum is calculted.
% Kolmogorov distance is a maximum of an absolute difference between 
% two cumulative distribution functions. 
% The number of objects in the output dataset D is equal to the one of DATA.
% The number of features in D is equal to the number of objects in PROTO.
% If PROTO is not supplied then PROTO==DATA.

% Copyright: Pavel Paclik, pavel@ph.tn.tudelft.nl
% Faculty of Applied Physics, Delft University of Technology
% P.O. Box 5046, 2600 GA Delft, The Netherlands

% $Id: dkolmogorov.m,v 1.3 2004/03/03 10:16:56 pavel Exp $

function d=dkolmogorov(data,proto)

%fprintf(' dkolmogorov:');

if ~exist('proto','var'), proto=data; end

if size(data,2)~=size(proto,2)
  error('both datasets must have the same feature sizes');
end
sc=size(data,1); % training sample size
pc=size(proto,1); % test sample size
if sc<2, errot('at least two samples are required'); end
if pc<2, errot('at least two samples are required'); end

labdata=getlab(data);
labproto=getlab(proto);
data2=data; data=+data;
proto2=proto; proto=+proto;

data=(cumsum(data'))';
proto=(cumsum(proto'))';

d=zeros(sc,pc);
%step=round(pc/10);
for i=1:pc
  t=repmat(proto(i,:),sc,1);
  d(:,i)=max(abs(data-t),[],2);
end
% d=dataset(d,labdata,'featlab',labproto);
d=setdata(data2,d,getlab(proto2));
      
return
  
