%DLH_TRUE likelyhood distance
%
%     D=DLH_TRUE(DATA,PROTO,DELTA)
%
% INPUT
%   DATA   a dataset
%   PROTO  the dataset with prototypes (representation set)
%   DELTA  reg parameter (by defautl =0)
% OUTPUT
%   D      dataset with a distance matrix
%
% DESCRIPTION
% DATA and PROTO preprocessed in such a way that 
% sum(DATA(i,:)) = sum(PROTO(j,:)) = 1
%  
% D computed as: 
% D(i,j) = - DATA(i,:)*log(PROTO(j,:))'
%
% if DATA(i,k) == 0 then DATA(i,k)*log(PROTO(j,k)) 
% considered as equal to zero
%

% $Id: dlh_true.m,v 1.7 2005/04/08 13:44:50 serguei Exp $

function d=dlh_true(data,proto,delta)

fprintf(' dlh_true:');

if nargin < 3
  delta = 0;
end

if delta < 0
  error('regularization parameter delta is negative');
end

nb = size(proto,2);
if size(data,2)~=nb
  error('both datasets must have the same feature sizes');
end

ns=size(data,1);  % the number of sampes
np=size(proto,1); % the number of prototypes

labdata=getlab(data);
labproto=getlab(proto);

data2=data; 
data=+data;

%proto2=proto; 
proto=+proto;

% normalization: we consider a histogram being a set of probability
% estimates
data=data./repmat(sum(data,2),[1,nb]);
proto=proto./repmat(sum(proto,2),[1,nb]);
if delta > 0
  proto = (1-delta)*proto + delta/nb;
  if ~isfinite(log(delta/nb))
    prwarning(1,'regularization parameter too small to prevent infinite distances');
  end
end

%if delta > 0
%  proto = (1-delta)*proto + repmat((delta/nb)*sum(proto,2),[1,nb]);
%end
%proto=proto./repmat(sum(proto,2),[1,nb]);

w = warning;
warning('off');
proto = log(proto);
warning(w);  

%if delta > 0 & isfinite(log(delta/nb))
%  d = data*proto';
%  d=setdat(data2,-d);
%  d=setfeatlab(d,labproto);
%  return
%end

d=zeros(ns,np);
if np <= ns
  zind = find(data == 0);
  for j=1:np
    product = data.*repmat(proto(j,:),[ns, 1]);
    product(zind) = 0;
    d(:,j) = sum(product,2);
  end
else  
  for i=1:ns
    zind = find(data(i,:) == 0);
    product = repmat(data(i,:),[np, 1]).*proto;
    product(:,zind) = 0;
    d(i,:) = sum(product,2)';
  end
end

d=setdat(data2,-d);
d=setfeatlab(d,labproto);

return
  
