%DQUADFORM Quadratic-form distance
%
%  D=DQUADFORM(DATA,PROTO)
%
% INPUT
%  DATA   testing dataset
%  PROTO  training dataset (representation set)
%
% OUTPUT
%  D      dataset with a distance matrix
%
% DESCRIPTION
% In these routine each spectrum is treated as a density of some distribution (histogram).
% The output dataset D contains the quadratic-form distances between such histograms from
% DATA and PROTO datasets. 
% Quadratic-form distance between two histograms h and k is
% sqrt((h-k)'*A*(h-k)) where h and k are columns and A(i,j) = 1-d(i,j)/dmax
%   d(i,j) is the absolute distance between bins (wavelengthes in our case) i and j 
%   dmax is the maximum d(i,j)
%
% The number of objects in the output dataset D is equal to the one of DATA.
% The number of features in D is equal to the number of objects in PROTO.
% If PROTO is not supplied then PROTO==DATA.

% Copyright: Pavel Paclik, pavel@ph.tn.tudelft.nl
% Faculty of Applied Physics, Delft University of Technology
% P.O. Box 5046, 2600 GA Delft, The Netherlands

% $Id: dquadform.m,v 1.4 2005/02/07 14:42:50 pavel Exp $

function d=dquadform(data,proto)

if ~exist('proto','var'), proto=data; end

if size(data,2)~=size(proto,2)
  error('both datasets must have the same feature sizes');
end
sc=size(data,1); % training sample size
pc=size(proto,1); % test sample size

fc=size(data,2);

a=abs(repmat(0:(fc-1),fc,1)-repmat((0:(fc-1))',1,fc));
a=1-a./(fc-1);

labdata=getlab(data);
labproto=getlab(proto);
data2=data; data=+data;
proto2=proto; proto=+proto;

d=zeros(sc,pc);
for i=1:pc
  t=data-repmat(proto(i,:),sc,1);
  d(:,i)=sum((t*a).*t, 2);
end

d=setdata(data2,d,labproto);
  
return
  
