%DSAM Spectral Angle Mapper (normalized inner product)
%
%  D=DSAM(DATA)
%  D=DSAM(DATA,PROTO)
%
% INPUT 
%  DATA    testing dataset
%  PROTO   training dataset (representation set)
% 
% OUTPUT 
%  D       dataset with a distance (normalized inner products) matrix
%
% DESCRIPTION   
% Variables in the output dataset D are normalized inner products
% between DATA samples and PROTO samples. The number of samples in D is equal to the 
% one of  DATA. D.OBJSIZE is the same as DATA.OBJSIZE. If PROTO is not supplied then
% PROTO==DATA.

% Copyright: Pavel Paclik, pavel@ph.tn.tudelft.nl
% Faculty of Applied Physics, Delft University of Technology
% P.O. Box 5046, 2600 GA Delft, The Netherlands

% $Id: dsam.m,v 1.7 2005/05/10 20:08:46 serguei Exp $

function d=dsam(data,proto)

if ~exist('proto','var'), proto=data; end

if size(data,2)~=size(proto,2)
  error('both datasets must have the same feature sizes');
end

v=size(data,2);
s=size(data,1);
p=size(proto,1);


%labdata  = getlab(data);
labproto = getlab(proto);

d = data;
d = setdata(d,zeros(s,1));

data  = +data;
proto = +proto;

if 0
step = 300;

b = 1:step:s;
e = step:step:s;
if length(e) < length(b)
  e = [e s];
end  

for i=1:length(b)
  data(b(i):e(i),:) = data(b(i):e(i),:)./repmat(sqrt(sum(data(b(i):e(i),:).^2,2)), [1 v]);
end

b = 1:step:p;
e = step:step:p;
if length(e) < length(b)
  e = [e p];
end  

for i=1:length(b)
  proto(b(i):e(i),:) = proto(b(i):e(i),:)./repmat(sqrt(sum(proto(b(i):e(i),:).^2,2)), [1 v]);
end
d = setdata(d, real(sqrt(1-min((data*proto'),1))), labproto);
end

ndata=sqrt(sum(data.*data,2));
nproto=sqrt(sum(proto.*proto,2));
d = setdata(d, real(sqrt(1-min((data*proto')./(ndata*nproto'),1))), labproto);

return
  
