%GETLI Return label image from a dataset or label vector
%
%  LI = GETLI(A,B)
%
% INPUT
%  A    dataset, label vector or several collated label vectors
%  B    if A is a label vector, B should be a dataset; size of the image defined by the B.OBJSIZE
%
% OUTPUT
%  LI   label image 
%
% DESCRIPTION
% Converts a dataset, a label vector or several collated label vectors
% into a dip image (or matrix, if DIPimage is not installed).

% Copyright: Pavel Paclik, pavel@ph.tn.tudelft.nl
% Faculty of Applied Physics, Delft University of Technology
% P.O. Box 5046, 2600 GA Delft, The Netherlands

% $Id: getli.m,v 1.7 2005/02/10 15:24:07 pavel Exp $

function li=getli(a,b)
	
	if isdataset(a)
	
	   objsize=a.objsize;
	   
	   if length(objsize)==1
	      
	      % the dataset does not contain image
	      % we can still return the labels reorganised
	      % into a square
	      
	      sc=size(a,1); % sample count
	      
	      t=ceil(sqrt(sc));
	      t2=ceil(sc/t);
	      
	      objsize=[t t2];
	      
	      li=zeros(t,t2);
	      li(1:sc)=a.nlab;
	      li(sc+1:end)=NaN;
	      
	   elseif length(objsize)==2
	      % classical 2D image stored in a dataset
	      li=reshape( a.nlab, objsize(1), objsize(2) );
	   else
	      error('unknown objsize');
	   end
	   
	elseif isnumeric(a) 
		t=sort(size(a));
		if t(1)>1
			li=a;
		elseif exist('b','var') & isdataset(b) & length(b.objsize)==2
	   
			% we have a label vector and a dataset defining its objsize
			li=reshape( a, [b.objsize, size(a,2)] );
		else
			error('dataset defining image size is required');
		end
	end

	if isdipinstalled
	   li=dip_image(li);
	else
	   prwarning(2,'DIPimage not installed, returning matrix');
	end
	
	return
	
