% GLDBM_FEATSEL Select bases in GLDB mapping using a feat.sel. mapping
%
%    W=GLDBM_FEATSEL(WGLDB,WFEATSEL)
%
% INPUT
%    WGLDB     GLDB mapping
%    WFEATSEL  feature selection mapping
%
% OUTPUT
%    W         reduced GLDB mapping
%

function w=gldbm_featsel(wgldb,wfeatsel)

   if iscell(wgldb) & iscell(wfeatsel)

      % We address the cell array using a linear index.
      % In the end, we reshape the cell array into the original
      % dimensions. 2D arrays (pair-wise) are supported.
      for i=1:prod(size(wgldb))
	 eval(sprintf('w{i}=%s(wgldb{i},wfeatsel{i});',mfilename));
      end
      w=reshape(w,size(wgldb));
% $$$       
% $$$       % call this function recursively for each class pair
% $$$       w={};
% $$$       for i=1:size(wgldb,1)
% $$$ 	 for j=i+1:size(wgldb,2)
% $$$ 	    eval(sprintf('w{%d,%d}=%s(wgldb{i,j},wfeatsel{i,j});',...
% $$$ 			 i,j,mfilename));
% $$$ 	 end
% $$$       end
      return
   end

   % for nested calls: for empty inputs return empty cell
   if isempty(wgldb) & isempty(wfeatsel)
      w=[];
      return
   end
   
   if ~ismapping(wfeatsel) | isempty(findstr('Feature Selection',getname(wfeatsel)))
      error('Trained feature selection mapping is required');
   end

   if iscell(+wfeatsel)
       % using cmapm feature selection mapping
       ind=wfeatsel{2};
   elseif isnumeric(+wfeatsel)
       % using featsel mapping
       ind=+wfeatsel; % selected features (== GLDB bases)
   else
       error('unknown feature selection mapping')
   end
   
   t=+wgldb;
   tout.clf=t.clf(ind);
   tout.l=t.l(ind);
   tout.u=t.u(ind);
   
   s=struct(wgldb);
   
   w=mapping('gldbm','trained',tout,...
	     s.labels,s.size_in,length(tout.l));
   return