% GLDBM_MC_FEATSEL Apply the feature selection to a multi-class GLDB mapping
%
%   W=GLDBM_MC_FEATSEL(WGLDB,WFEATSEL)
%
% INPUT
%    WGLDB     GLDB mapping
%    WFEATSEL  feature selection mapping
%
% OUTPUT
%    W         reduced GLDB mapping
%
% DESCRIPTION
% Results of a multi-class GLDB criterion and a follow-up feature selection
% are merged into a single GLDB mapping.

function w=gldbm_mc_featsel(wgldb,wfeatsel)
	
	tw=+wgldb;
	featind=+wfeatsel;
	if ~isnumeric(featind)
	    featind=featind{2}; % indices of selected features 
	end
	
	% group indices: output features mapped to gldb groups
	% 111233
	% 3 features from group 1, 1 from 2, 2 from 3 etc.
	groupc=size(tw.clf,2);
	temp=zeros(1,groupc);
	for i=1:groupc
	    temp(i)=size(tw.clf(i).w,2); % output size of the group
	end
	fout=genlab(temp);

        groups=zeros(1,groupc); % flag: is the info for this output group?
	gdata=[];

	for i=1:length(featind) 

	    % extract the group
	    g=fout(featind(i)); % the group of feature featind(i)

	    W=tw.clf(g).w; % projection matrix
	    
	    % binary map of selected features inside fout
	    binmap=zeros(1,length(fout));
	    binmap(featind(i))=1; % mark the feature we're interested in
	    gind=find(fout==g); % get the indices of the group's features
	    temp=binmap(gind); % get the part of the binary map belonging
                               % to the group we're dealing with
	    ind=find(temp); % ind should be the index of our feature inside our group
	    if isempty(ind), error(' feature not found in the group'); end

	    % get only the column corresponding to the ind-th feature
	    W=W(:,ind);
	    
	    if ~groups(g) % the group does not exist yet
		gdata(g).clf=W;
		gdata(g).l=tw.l(g);
		gdata(g).u=tw.u(g);
		groups(g)=1; % mark as existing
	    else
		gdata(g).clf=[gdata(g).clf W];
		% the band boundaries do not change!
	    end
	end

	% output gldb mapping
	tout.clf=[];
	tout.l=[];
	tout.u=[];

	outd=0; % output dimensionality
	% construct output gldb structure
        gind=find(groups); % existing groups
        for i=1:sum(groups)
	    tout.clf(i).w=gdata(gind(i)).clf; 
	    outd=outd+size(gdata(gind(i)).clf,2);
	    tout.l(i)=gdata(gind(i)).l;
	    tout.u(i)=gdata(gind(i)).u;	   
	end

	s=struct(wgldb);
	
	w=mapping('gldbm','trained',tout,...
		  s.labels,s.size_in,outd);
	
