%OPAPS Orthogonal projection approach: selects "pure" objects
%  
%
%  [B, SEL_OBJ_IND, SEL_OBJ_DIS, DIS, ERR] = OPAPS(A)
%  [B, SEL_OBJ_IND, SEL_OBJ_DIS, DIS, ERR] = OPAPS(A, OPT)
%
%
% INPUT 
%   A             [M -by- K] training dataset
%
%   OPT.   
%    MAX_NUM      the max number of objects to select, INF means selecting as many objects as possible,
%                 until the rank of dataset is exausted; at least one variable will be always selected; by delault = INF
%    EPS          stopping criteion; algorithm stops ERR < EPS, such objects are not included into the set; by default = 1e-2
%    VERBOSE      force/disable messages: PRPROGRESS(VERBOSE,['%s\n'],MSG) will be run; 
%                 by default = []
%
% OUTPUT 
%   B              [M_SEL -by- K] dataset containing selected object
%   SEL_OBJ_IND    [M_SEL -by- 1] vector of selected objects indices 
%   SEL_OBJ_DIS    [M_SEL -by- 2] two vectors of selected objects dissimilarities 
%                  in the first column are absolute dissims, and in the second are relative dissim
%   DIS            [M -by- M_SEL -by- 2] two matrices which columns are dissimilarities profiles; 
%                  each columns contains dissimilarities of all objects at particular selection step;
%                  the first matrix contains abs dissim, the second one containes relative dissim
%   ERR            [M_SEL -by- 1] relative squared error of the data projection to the subspace spanned by the select objects
%
% DESRIPTION
% Implementation of the OPA algorithm as described in
% "Orthogonal Projection Approach Applied to Peak Purity Asessment"  
%  by F. C. Snachez, J. Toft, B. van den Bogaert, D. L. Massart, Anla. Chem. 68, 1, 79-85, 1996 
%
% Absolute dissim measure  abs_diss_(n+1) = ||dx||^2*det(Y_n Y_n^T)
% Relative dissim measure  rel_diss_(n+1) = ||dx||^2
%
% Here, Y_n is the set of n selected objects (rows), which are normalized to have a unit length, dx is the part of 
% an object x which is (part) orthogonal to span(Y). If n=0 (diss(1)) the dissimilarity to the mean object is measured.
%
% SEE ALSO
% SIMPLISMAPS, VARIMAXOM, SIPMLISMAM, OPAM, VARIMAXOM, ALS, OPA



function [b, sel_obj_ind, sel_obj_dis, dis, err] = opaps(a, Opt)

if nargin < 2
  Opt = [];
end

[y, sel_obj_ind, sel_obj_dis, dis, err] = opa(+a,Opt);

b = a(sel_obj_ind,:);
b.name = [b.name ': selected spectra'];

return

