%pca_stat Calculation statistics on the results of PCA
%
%  Stat = pca_stat(X,T,P,lambda,nPC_model,Options)
%
% INPUT 
%   X         [N -by- d]   data matrix, N samples, d variables
%   T         [N -by- nPC] scores of X (returned by pca_apply or by pca_train)
%   P         [N -by- nPC] loadings (returned by pca_train)
%   lambda    [1 -by- nPC]  first nPC largest eigenvalues 
%   nPC_model the number of PCs in the model for which statistics will be calculated
%             (nPC_model <= nPC)
%   Options   structure returned by pca_train
%
% OUTPUT
%  Stat.   the structure containig statistical information calculated
%    Q       [N -by- 1] Euclidian distances to the model plane
%    T2      [N -by- 1] Mahalanobis distances in the model plane
%    T2_rest [N -by- 1] Mahalanobis distances out the model plane
%
% DESCRIPTION
%
% SEE ALSO
% pca_apply

% Copyright: S. Verzakov, serguei@ph.tn.tudelft.nl
% Faculty of Applied Sciences, Delft University of Technology
% P.O. Box 5046, 2600 GA Delft, The Netherlands

% $Id: pca_stat.m,v 1.3 2005/05/05 14:35:55 serguei Exp $



function Stat = pca_stat(X,T,P,lambda,nPC_model,Options)

if nPC_model > length(lambda)
  error('The number of PCs in the model is large than the number of PC availabel (nPC_model > length(lambda))');
end

[N,d] = size(X);

X = prepro(X,Options.X_centering,Options.X_scaling);

Stat.Q  = sum((X-T(:,1:nPC_model)*P(:,1:nPC_model)').^2,2);
Stat.T2 = sum((T(:,1:nPC_model).^2)./ repmat(lambda(1:nPC_model), [N 1]),2);

if nPC_model < Options.nPC
  Stat.T_rest2 = sum((T(:,(nPC_model+1):end).^2)./ repmat(lambda((nPC_model+1):end), [N 1]),2);

else
  WarnMsg = sprintf(['The number of PC''s availabel is equal to the nubmer of the PC''s in model.\n'...
                     'Stat.T2_rest cannot be calculated, set to nan\n']);
  if exist('prwarning')
    prwarning(2, WarnMsg);
  else
    warning(WarnMsg);
  end
  Stat.T2_rest = repmat(nan,[N,1]);  
end

return;

