% PLOT_CCA Superimpose training and test data projected by CCA
%
%    PLOT_CCA(TR,TS)
%
% INPUT
%   TR    training set
%   TS    test set
%
% DESCRIPTION
% This routine visualises differences between training and test sets.
% Canonical correlation mapping is computed on the training dataset.  Both
% training and testing dataset are projected into the factor space by the
% trained mapping and the scatter plots are displayed. Examples in the
% training set are drawn with darker and thicker markers than in the test.

% $Id: plot_cca.m,v 1.3 2005/11/02 12:35:09 pavel Exp $

% Copyright: P. Paclik, pavel@ph.tn.tudelft.nl
% Faculty of Applied Physics, Delft University of Technology
% P.O. Box 5046, 2600 GA Delft, The Netherlands

function plot_cca(tr,ts)
	
	% compute the cca mapping
	w=ccam(tr);
	
	% plot the test data projected in the factor space
	fh=scatterdui(ts*w);

	hh=findobj(fh,'type','axes');
	hts_classes=findobj(hh,'type','line');
	
	hold on
	
	% plot the projected training data
	fh=scatterd(tr*w);
	htr_classes=findobj(fh,'type','line');
	for i=1:length(htr_classes)
	    col=get(htr_classes(i),'color');
	    set(htr_classes(i),'color',0.6*col); % darken
	end
	set(htr_classes,'markersize',10,'linewidth',3);

	
	return
	