% PLOT_GLDB_GROUPS Plot groups selected by GLDB
%
%   PLOT_GLDB_GROUPS(A,W)
%   plot_gldb_groups(a,L,H)
%
% INPUT
%   A   - spectral dataset
%   W   - trained GLDB mapping
%   L,H - vectors with lower-upper indices of bands forming groups
%
% DESCRIPTION
% This routine plots the first spectrum from the supplied dataset A
% and highlights all groups of bands, defined by the G:DB mapping W.
% Successive groups are denoted by changing color.
%
% SEE ALSO
% GLDB, GLDBM_MULTI NLGLDBM

function plot_gldb_groups(a,par1,par2,par3)

   if isa(par1,'mapping')
      l=par1.data.l;
      u=par1.data.u;
      if exist('par2','var')
	 t=par2+abs(min(par2));
	 crit=t./sum(t);
	 %      colormap(colmap);
      end
   else
      l=par1; u=par2;
      if exist('par3','var')
	 t=par3+abs(min(par3));
	 crit=t./sum(t);
	 %      colormap(colmap);
      end
   end
   clf
   colmap=colormap; % default map
   
   col='g';
   for i=1:length(l)
      
      bands=l(i):u(i);
      values=+a(1,bands);
      bands=[l(i)-0.5 bands u(i)+0.5];
      values=[values(1) values values(length(values))];
      h=patch([max(bands) min(bands) sort(bands)],[0 0 values],col);

      if exist('crit','var')
	 ind=min(find(crit>=crit(i)));
	 set(h,'FaceColor',colmap(end-ind,:));
      else
	 if strcmp(col,'g')
	    col='k';
	 else
	    col='g';
	 end
      end
   end
   
   return
   