% PLOTS Plot spectra
%
%  PLOTS(A,COL)
% 
% INPUT
%  A    dataset
%  COL  linecolor-symbol-linestyle (e.g. 'r:--')
% 
% DESCRIPTION
% This routine plots spectra from dataset A. It uses information from
% A.USER field to put correct values on the X (wavelengths/wavenumbers)
% axis.  If COL parameter is not supplied then automatically selected
% linecolor-symbol-linestyle will be used to plot spectra of each class (in
% the case of labelled dataset)

% Copyright: Pavel Paclik, pavel@ph.tn.tudelft.nl
% Faculty of Applied Physics, Delft University of Technology
% P.O. Box 5046, 2600 GA Delft, The Netherlands

% $Id: plots.m,v 1.12 2005/11/02 14:56:31 pavel Exp $

function plots(a,col)

    colors={'b','r','k','g','m','b--','r--','k--','g--','m--','b-.','r-.', ...
	    'k-.','g-.','m-.','b:','r:','k:','g:','m:','b+','r+','k+','g+','m+','bo','ro','ko','go','mo'};

    if isa(a,'dataset')
	lab=renumlab(getlab(a));
	cls=unique(lab);
	if length(cls)==1 & cls(1)==0
	    lab=ones(1,size(a,1));
	    cls=1;
	end
    else
	lab=ones(1,size(a,1));
	cls=1;
    end

    a=+a;
    hold on
    if exist('col','var')
	for i=1:size(a,1)
	    plot(a(i,:),col);
	end
    else
	for i=1:size(a,1)
	    plot(a(i,:),colors{lab(i)});
	end
    end

    return
