% PROTOSELM Prototype selection mapping
%
%   W=PROTOSELM(A,TYPE,PAR)
%
% DESCRIPTION
% This function implements traineble prototype selection mapping.  It is
% applied to a dataset with dissimilaritites. Once trained, it acts
% as simple feature selection mapping.
% 
% types:
%  rand - random selection of PAR prototypes from all data samples
%  randcls - random selection using PAR samples per class
%  

% $Id: protoselm.m,v 1.2 2005/02/10 15:24:07 pavel Exp $

function w=protoselm(a,type,par)

	if (nargin == 0)
		w = mapping(mfilename,{'rand',1});
		w = setname(w,['Prototype selection mapping']);
		return
	end
	
	% allow simple construction of untrained mapping:
	if nargin==1 & ischar(a), type=a; a=[]; end 
	if isempty(a) & ischar(type)
		if ~exist('par','var'), par=1; end
		w = mapping(mfilename,{type,par});
		w = setname(w,['Prototype selection mapping ''' type '''']);
	end
	
	if isdataset(a) & ~ismapping(type)
		% train the dissimilarity mapping
		[m,k,c] = getsize(a);
		if ~exist('par','var'), par=[]; end

		% for each method, set proto to indices of prototypes
		switch type
			
		 case {'rand', 'random'}
		  % randomly select prototypes
		  % par: number of prototypes
		  ind=randperm(size(a,2));
		  proto=ind(1:par);
		  
		 case 'randcls'
		  % select prototypes randomly from each class
		  % par: number of prototypes per class
		  [temp,temp2,proto]=gendat(a,par*ones(1,c));
		  
		 otherwise
		  error('unknown type');
		  
		end
		w = mapping(mfilename,'trained',{type,par,proto},a.lablist,k,length(proto));
		w = setname(w,'Prototype selection mapping');
	end
	
	if isdataset(a) & ismapping(type)
		% execute the mapping
		par=type{2};
		proto=type{3};
		w=setdat(a,a(:,proto));
	end

	return
	
