% RANDFEATSELM Random feature selection with validation
%
%      [W,P,E]=RANDFEATSELM(DATA,N,D,WIN)
%
% INPUT
%   DATA   input dataset
%   N      number of randomly generated feature subsets
%   D      requested dimensioanlity
%   WIN    untrained mapping
%
% OUTPUT
%   W      feature selection mapping with the best subset found
%   P      the final population (uint8 matrix, each row is a
%          binary-encoded feature subset)
%   E      error rate for each feature subset
%
% DESCRIPTION
% This function generates randomly N feature subsets of dimensionality D.
% For each subset a mapping WIN is traned and an apparent error rate is
% used as criterion. Subsets are sorted according to the error and the
% feature selection mapping with the best subset is returned as W.
% Feature selection mappings for subset i may be retrieved using:
%   w2=cmapm(size(data,2),find(P(i)==1))

% $Id: randfeatselm.m,v 1.3 2005/02/10 15:24:07 pavel Exp $

function [w,P,e]=randfeatselm(data,N,D,win)
	
	fc=size(data,2);
	
	P=create_init_pop(N,fc,D);
	e=get_err(data,P,win); % population errors

	[s,ind]=sort(e);
	e=e(ind);
	P=P(ind,:);
	
	w=cmapm(fc,find(P(1,:)==1));	
	
	return
	
% N - number of individuals to create
% d - number of dimensions (features) we search in
% dim - how many dimensions should be switched on
%       Dim may be a vector specifying lower and upper bound)
function P=create_init_pop(N,d,dim)

	% population matrix
	P=repmat(uint8(0),N,d);

	[ignore,ind]=sort(rand(N,d),2);
	clear ignore
	
	% feature count per individual:
	if length(dim)==1
		% constant number of switched on-dimensions
		fc=dim*ones(N,1);
	else
		fc=dim(1)+floor(rand(N,1)*dim(2));
	end

	for i=1:N
		try
			P(i,ind(i,1:fc(i)))=1;
		catch
			keyboard
		end
	end
	
	return
	
% measure the error for each feature setup (chromosome) in P
% on data using user-defined mapping w
function e=get_err(data,P,win)
	
	e=zeros(size(P,1),1);
	for i=1:size(P,1)
		ind=find(P(i,:)>0);
                w2=data(:,ind)*win;
		e(i)=data(:,ind)*w2*testc;
	end
	
	return
	