%SAMC Spectral Angle Mapper classifier
% 
%   W = SAMC(A)
% 
% INPUT  
%   A    Training dataset
%
% OUTPUT
%   W    Spectral Angle Mapper mapping
%
% DESCRIPTION
% Classifier based on SAM similarity measure. A mean spectrum of each
% class is considered a prototype. In order to use more prototypes per
% class,
%
% SEE ALSO
% DATASETS, MAPPINGS

% $Id: samc.m,v 1.2 2005/02/10 10:00:54 pavel Exp $

function w = samc(a,arg2)

	% No arguments given: return untrained mapping.

	if (nargin < 1) | (isempty(a))
		w = mapping(mfilename,{arg2});
		w = setname(w,'Spectral Angle Mapper classifier');
		return
	end

	if (~exist('arg2','var'))     % Second argument is not a mapping: training.

		isdataset(a);           % Assert that A is a dataset.

		[m,k,c] = getsize(a); M = classsizes(a);

		% class means
		pars.means=meancov(a);
		
		% Save all useful data.

		w = mapping(mfilename,'trained',pars,getlablist(a),k,c);
		w = setname(w,'Spectral Angle Mapper classifier');

	else                      % Second argument is a a mapping: testing.

		w = arg2;
		pars = getdata(w);		 	% Unpack the mapping.
		[m,k] = getsize(a); 

		c = size( pars.means, 1 );

		% computing x'x
		t=+a;
		X=zeros(size(t,1),1);
		for i=1:size(t,1)
			X(i)=t(i,:) * t(i,:)';
		end

		% class means
		means=+pars.means;
		M=zeros(size(pars.means,1),1);
		for i=1:c
			M(i)=means(i,:) * means(i,:)';
		end
		
		X= sqrt(X) * sqrt(M)';
		
		% we output the goodness to fitt the class = the higher, the
                % better
		out=(pi/2)-acos( (t * means')./X );

		w = setdat(a,out,w);

	end

return



