% SHOWSI_UPDATE - callback functions for showsi
%

% Copyright: Pavel Paclik, pavel@ph.tn.tudelft.nl
% Faculty of Applied Physics, Delft University of Technology
% P.O. Box 5046, 2600 GA Delft, The Netherlands

function showsi_update(arg1,arg2)

 fig=gcbf;

 
 switch lower(arg1)

  case 'down_dip_area1'
   % area mode: 1st stage
   % when clicked, we color the current spectrum and switch to the
   % 2nd stage where range is defined in the spectral figure

   if strncmp(get(fig,'Tag'),'DIP_Image',9)
    img = get(fig,'CurrentObject');
    if strcmp(get(img,'Type'),'image')

     udim = get(fig,'UserData');
     udpim = get(udim.hpim,'UserData');
     udspec=get(udim.hspec,'UserData');

     % by the left button, we draw the spectrum under the pointer
     udim.si_ax = get(img,'Parent');
     udim.oldAxesUnits = get(udim.si_ax,'Units');
     udim.oldNumberTitle = get(fig,'NumberTitle');
     set(udim.si_ax,'Units','pixels');

     pt = dipfig_getcurpos(udim.si_ax);	

     % get the spectrum from the primary image
     data=double(squeeze(udpim.slices(pt(1),pt(2),:)));

     % update the prepared line in the spec plot
     set(udspec.si_templine,'ydata',data);
     
     % change the color to indicate the mode change
     set(udspec.si_templine,'color',[0 0 1]);

     % second mode, no motion tracked in pim window
     set(fig,'WindowButtonDownFcn','showsi_update(''down_dip_area2'')');
     set(fig,'WindowButtonMotionFcn','');	
     % area2 mode for plotting points on the spectrum
     set(udpim.hspec,'WindowButtonMotionFcn','showsi_update(''motion_spec_area2'')');	
     set(udpim.hspec,'WindowButtonDownFcn','showsi_update(''down_spec_area2'')');	

     % draw the first points
     set(udspec.si_area_line,'visible','on');
     %XXX
     %set(udim.si_ax,'Units',udim.oldAxesUnits);
     set(udim.si_ax,'Units','normalized');
    end
   end  
   
   
  case 'down_dip_area2'
   % area mode: 2nd stage
   % when right-button is clicked, we return back to the 1st stage

   if strncmp(get(fig,'Tag'),'DIP_Image',9)
    img = get(fig,'CurrentObject');
    if strcmp(get(img,'Type'),'image')

     if strcmp(get(fig,'SelectionType'),'alt')
     
      % right button -> go back to the 1st stage
      udim = get(fig,'UserData');
      udpim = get(udim.hpim,'UserData');
      udspec=get(udim.hspec,'UserData');

      % change the color to indicate the mode change
      set(udspec.si_templine,'color',[0.5 0.5 0.5]);
      
      set(fig,'WindowButtonDownFcn','showsi_update(''down_dip_area1'')');
      set(fig,'WindowButtonMotionFcn','showsi_update(''motion_dip'')');	
      % switch off area more
      set(udpim.hspec,'WindowButtonMotionFcn','');	
      set(udspec.si_area_line,'visible','off');
      
     end
    end
   end

  case 'motion_spec_area2'
   
   % moving the first points along the spectral curve
   
   udspec = get(fig,'UserData');
   udpim = get(udspec.hpim,'UserData');

   set(udspec.si_ax,'Units','pixels');
   pt = dipfig_getcurpos(udspec.si_ax);	

   data=get(udspec.si_templine,'ydata');

   [m,ind]=min(abs(udspec.si_bands-pt(1)));

   set(udspec.si_area_line,'xdata',[pt(1) pt(1)]);
   set(udspec.si_area_line,'ydata',[data(ind) data(ind)]);
   
   set(fig,'UserData',[]);
   set(fig,'UserData',udspec);
   
   %XXX
   set(udspec.si_ax,'Units','normalized');

  case 'down_spec_area2'
   % area mode: 2nd stage
   % left button defines the first point and switches to the 3rd stage
   % (definition of the second point)

   if ~strcmp(get(fig,'SelectionType'),'alt')
     
      udspec=get(fig,'UserData');
      udpim = get(udspec.hpim,'UserData');

      set(udspec.si_area_line,'color',[1 0 0]);
      
      set(fig,'WindowButtonDownFcn','showsi_update(''down_spec_area3'')');
      set(fig,'WindowButtonMotionFcn','showsi_update(''motion_spec_area3'')');	
      
   end

  case 'motion_spec_area3'
   
   % moving the 2nd point and line
   
   udspec = get(fig,'UserData');
   udpim = get(udspec.hpim,'UserData');

   set(udspec.si_ax,'Units','pixels');
   pt = dipfig_getcurpos(udspec.si_ax);	

   data=get(udspec.si_templine,'ydata');

   [m,ind]=min(abs(udspec.si_bands-pt(1)));

   x=get(udspec.si_area_line,'xdata');
   y=get(udspec.si_area_line,'ydata');
   set(udspec.si_area_line,'xdata',[x(1) pt(1)]);
   set(udspec.si_area_line,'ydata',[y(1) data(ind)]);
   
   set(fig,'UserData',[]);
   set(fig,'UserData',udspec);

   %XXX
   set(udspec.si_ax,'Units','normalized');

  case 'down_spec_area3'
   % area mode: 3rd stage
   % left button defines the second point 
   % right button returns to the 2nd stage

   udspec=get(fig,'UserData');
   udpim = get(udspec.hpim,'UserData');
   x=get(udspec.si_area_line,'xdata');
   y=get(udspec.si_area_line,'ydata');
   
   if strcmp(get(fig,'SelectionType'),'alt')

    set(udspec.si_area_line,'xdata',[x(1) x(1)]);
    set(udspec.si_area_line,'ydata',[y(1) y(1)]);
    
    set(udspec.si_area_line,'color',[0 0 1]);
      
    set(udpim.hspec,'WindowButtonMotionFcn','showsi_update(''motion_spec_area2'')');	
    set(udpim.hspec,'WindowButtonDownFcn','showsi_update(''down_spec_area2'')');	
    
   else
    % left button defines second point
    set(udspec.si_area_line,'color',[0 0 1]);
    
    set(fig,'WindowButtonDownFcn','showsi_update(''down_spec_area3'')');
    set(fig,'WindowButtonMotionFcn','');

    [m,ind1]=min(abs(udspec.si_bands-x(1)));
    [m,ind2]=min(abs(udspec.si_bands-x(2)));    
    % generate the area image
    im=chemimage(udpim.slices,[ind1 ind2]);
    imh=dipshow(figure,im,'percentile','notruesize');
    % change the colormap
    set(imh,'colormap',jet(256));
%    si_attach_display(udpim.hpim,imh);
   end
   
   
  case 'down_dip'
   % general dip visualization
   if strncmp(get(fig,'Tag'),'DIP_Image',9)
    img = get(fig,'CurrentObject');
    if strcmp(get(img,'Type'),'image')

     udim = get(fig,'UserData');
     udpim = get(udim.hpim,'UserData');
     udspec=get(udim.hspec,'UserData');

     if strcmp(get(fig,'SelectionType'),'alt')
      % by right button, we clear the axis from all tagged points
      h=findobj(get(udspec.si_ax,'children'),'type','line','tag','spectra');
      delete(h);
      % delete the points in the primary one
      h=findobj(get(udim.hpim,'children'),'type','line','tag','spectra');
      delete(h);
      
      % delete the points from all the attached images
      for i=1:length(udpim.si_attached_displays)
       h=findobj(get(udpim.si_attached_displays(i),'children'),...
		 'type','line','tag','spectra');
       delete(h);
      end

      % delete stored point coordinates
      udpim.si_points=[];
     else
      % by the left button, we draw the
      % spectrum under the pointer
      udim.si_ax = get(img,'Parent');
      udim.oldAxesUnits = get(udim.si_ax,'Units');
      udim.oldNumberTitle = get(fig,'NumberTitle');
      set(udim.si_ax,'Units','pixels');

      pt = dipfig_getcurpos(udim.si_ax);	

      % get the spectrum from the primary image
      data=double(squeeze(udpim.slices(pt(1),pt(2),:)));

      % update the list of points
      udpim.si_points=[udpim.si_points; pt(1), pt(2), udspec.si_class];
      
      % draw points also in the primary one
      figure(udim.hpim);
      h=line([pt(1) pt(1)],[pt(2),pt(2)]);
      set(h, 'color', udspec.si_classcolmap(udspec.si_class,:),...
	     'marker','.','tag','spectra');
      
      % draw the points in all attached images
      for i=1:length(udpim.si_attached_displays)
       figure(udpim.si_attached_displays(i));
       h=line([pt(1) pt(1)],[pt(2),pt(2)]);
       set(h, 'color', udspec.si_classcolmap(udspec.si_class,:),...
	      'marker','.','tag','spectra');
      end
      
      % draw the whole spectrum in the spec figure
      figure(udim.hspec);
      h=plot(udspec.si_bands,data,'color', udspec.si_classcolmap(udspec.si_class,:),...
	     'tag','spectra');
     
      %XXX
      %set(udim.si_ax,'Units',udim.oldAxesUnits);
      set(udim.si_ax,'Units','normalized');
     end

     set(fig,'UserData',[]);
     set(fig,'UserData',udim);

     set(udim.hpim,'UserData',[]);
     set(udim.hpim,'UserData',udpim);

     set(udim.hspec,'UserData',[]);
     set(udim.hspec,'UserData',udspec);
    end
   end

  case 'motion_dip'

   if strncmp(get(fig,'Tag'),'DIP_Image',9)
    img = findobj(fig,'type','image');
    if ~isempty(img)
     udim = get(fig,'UserData');
     udim.si_ax = get(img,'Parent');
     udim.oldAxesUnits = get(udim.si_ax,'Units');
     udim.oldNumberTitle = get(fig,'NumberTitle');
     set(udim.si_ax,'Units','pixels');
     pt = dipfig_getcurpos(udim.si_ax);	

     % get spectrum from the primary image
     udpim = get(udim.hpim,'UserData');
     objsize=size(udpim.slices);
     data=double(squeeze(udpim.slices(pt(1),pt(2),:)));
     
     % update the prepared line in the spec plot
     udspec = get(udim.hspec,'UserData');			  
     set(udspec.si_templine,'ydata',data);
     
     if fig ~= udim.hpim
      if isfield(udim,'curslice') % only for higher dim images
       % this image is an attached one
       set(fig,'Name',sprintf('%d (%d,%d) value: %5.3f, band: %d',...
			      objsize(2)*pt(1)+pt(2)+1,pt(1),pt(2), udim.curslice+1,data(udim.curslice+1) ));
      end
     else
      % this picture is a primary image (pim)
      set(fig,'Name',sprintf('%d (%d,%d) band:%d, val:%5.3f',...
			     objsize(2)*pt(1)+pt(2)+1,pt(1),pt(2), udim.si_bands(udim.curslice+1),data(udim.curslice+1)));
     end
     
     % possible scatterd updates
     if ~isempty(udpim.si_attached_scatters)
	     for i=1:length(udpim.si_attached_scatters)
		     fig=udpim.si_attached_scatters(i);
		     ud=get(fig,'userdata');

		     data=+ud.a( pt(1)*objsize(2)+pt(2)+1, ...
				 [ ud.xdim, ud.ydim ] );

		     obj=findobj(fig,'tag','fastupdate');
		     if isempty(obj)
			     % create the point object
			     figure(fig);
			     ud.si_point = plot([data(1) data(1)],[data(2) data(2)],'o','color',[0 0 0]);
			     set(ud.si_point,'markersize',9,'markerfacecolor',[1 1 0]);
			     set(ud.si_point,'tag','fastupdate');
		     else
			     % the point exists
			     set(obj,'visible','on','color',[0 0 0]);
			     set(obj,'xdata',[data(1),data(1)]);
			     set(obj,'ydata',[data(2),data(2)]);		     
		     end
	     
		     set(fig, 'Name',sprintf('id %d', pt(1)*objsize(2)+pt(2)+1));

		     set(fig,'UserData',[]);
		     set(fig,'UserData',ud);
	     end
     end
     %XXX
     %set(udim.si_ax,'Units',udim.oldAxesUnits);
     set(udim.si_ax,'Units','normalized');
    end
   end

  case 'down_spec'
   % visualization mode: move through bands
   udspec = get(fig,'UserData');
   udpim = get(udspec.hpim,'UserData');
   if udspec.si_movement==0
    udspec.si_movement=1;
    set(fig,'WindowButtonMotionFcn', 'showsi_update(''motion_spec'')');	
    set(udspec.si_ax,'Units','pixels');
    pt = dipfig_getcurpos(udspec.si_ax);	
    % points are in the range of bands, find corresponding index:
    [m,ind]=min(abs(udspec.si_bands-pt(1)));
    udpim.curslice=ind-1;	  
    set(udspec.si_cursliceline,'xdata',[ind ind]);
    dipmapping(udspec.hpim,'slice',udpim.curslice);

    set(fig,'UserData',[]);
    set(fig,'UserData',udspec);
    set(udspec.hpim,'UserData',[]);
    set(udspec.hpim,'UserData',udpim);
    %XXX
    set(udspec.si_ax,'Units','normalized');
   else
    udspec.si_movement=0;
    set(fig,'WindowButtonMotionFcn','');
    set(fig,'UserData',[]);
    set(fig,'UserData',udspec);
   end

  case 'motion_spec'
   
   udspec = get(fig,'UserData');
   udpim = get(udspec.hpim,'UserData');
   set(udspec.si_ax,'Units','pixels');
   pt = dipfig_getcurpos(udspec.si_ax);	% we copied the dip private function
   set(udspec.si_cursliceline,'xdata',[pt(1) pt(1)]);
   % points are in the range of bands, find corresponding index:
   [m,ind]=min(abs(udspec.si_bands-pt(1)));
   udpim.curslice=ind-1;	  
   % update dip_image display
   dipmapping(udspec.hpim,'slice',udpim.curslice);
   set(fig,'Name',sprintf(' band: %5.3f ',udspec.si_bands(udpim.curslice+1)));

   set(fig,'UserData',[]);
   set(fig,'UserData',udspec);
   set(udspec.hpim,'UserData',[]);
   set(udspec.hpim,'UserData',udpim);

   %XXX
   set(udspec.si_ax,'Units','normalized');
  
  %case 'rb'
   
   %udspec = get(fig,'UserData');	  
   %udspec.si_class=arg2;
   %eval(sprintf('set(udspec.si_rb%d,''value'',%d);',arg2,1));
   %others=setdiff(1:3,arg2);
   %for i=1:length(others)	
   % eval(sprintf('set(udspec.si_rb%d,''value'',%d);',others(i),0));
   %end
   
   %set(fig,'UserData',[]);
   %set(fig,'UserData',udspec);
   
  case 'lst'
   udspec = get(fig,'UserData');	  
   udspec.si_class=get(udspec.si_lst,'value');

   plotcolor = udspec.si_classcolmap(udspec.si_class,:);
   weights = [1; 1.5; 0.5]; 
   if sum(plotcolor*weights/sum(weights)) >= 0.5
     textcolor = [0 0 0];
   else
     textcolor = [1 1 1];
   end

   if 0
   %textcolor = 1 - repmat(mean(plotcolor),[1 3]);
   textcolor = rgb2hsv(1-plotcolor);
   textcolor(2) = 0.5*textcolor(2);
   textcolor = hsv2rgb(textcolor);
   end

   set(udspec.si_colindicator,'backgroundcolor',plotcolor,'foregroundcolor',textcolor);


   %eval(sprintf('set(udspec.si_rb%d,''value'',%d);',arg2,1));
   %others=setdiff(1:3,arg2);
   %for i=1:length(others)	
   % eval(sprintf('set(udspec.si_rb%d,''value'',%d);',others(i),0));
   %end
   
   set(fig,'UserData',[]);
   set(fig,'UserData',udspec);
   


  case 'close'
   ud = get(fig,'UserData');
   udpim = get(ud.hpim,'UserData');

   % update closereq callbacks of attached displays and scatters
   for i=1:length(udpim.si_attached_displays)
	   si_dettach_display(ud.hpim,udpim.si_attached_displays(i));
   end
   for i=1:length(udpim.si_attached_scatters)
	   si_dettach_display(ud.hpim,udpim.si_attached_scatters(i));
   end
   if fig==ud.hpim
    delete(ud.hspec);
   elseif fig==ud.hspec
    delete(ud.hpim);
   end
   delete(fig)
   
  case 'close_attached_display'
   ud = get(fig,'UserData');
   % it's an attached image: remove its handle from the list
   udpim = get(ud.hpim,'UserData');
   udpim.si_attached_displays=setdiff(udpim.si_attached_displays,fig);
   set(ud.hpim,'UserData',[]);
   set(ud.hpim,'UserData',udpim);
   delete(fig);

  case 'close_attached_scatter'
   ud = get(fig,'UserData');
   % it's an attached scatter: remove its handle from the list
   udpim = get(ud.hpim,'UserData');
   udpim.si_attached_scatters=setdiff(udpim.si_attached_scatters,fig);
   set(ud.hpim,'UserData',[]);
   set(ud.hpim,'UserData',udpim);
   delete(fig);
   
 end
