%SI_ATTACH_DISPLAY Attach an additional display to the spectral image
%
%  SI_ATTACH_DISPLAY(SIH,IMH)
%
% INPUT
%   SIH    spectral image handle (see showsi)
%   IMH    figure handle to be attached
%

% Copyright: Pavel Paclik, pavel@ph.tn.tudelft.nl
% Faculty of Applied Physics, Delft University of Technology
% P.O. Box 5046, 2600 GA Delft, The Netherlands

function si_attach_display(sih,imh)
	
	udpim=get(sih,'userdata');
	udim=get(imh,'userdata');

	% fill in mandatory fields
	udim.hpim=sih; % link to the primary image
	udim.hspec=udpim.hspec; % link to the spectral figure
	udim.si_ax = get(imh,'CurrentAxes'); % image's axes

	% what figure we attach?
	tag=get(imh,'tag');

	if strcmp(tag,'scatterdui')
	
		% it is the scatterdui figure

		% update the list of attached figures
		udpim.si_attached_scatters=[udpim.si_attached_scatters imh];
		
		% create the point object
		udim.si_point = plot([0 0],[0 0],'o','color',[0 0 0]);
		set(udim.si_point,'markersize',9,'markerfacecolor',[1 1 0]);
		set(udim.si_point,'visible','off');
		set(udim.si_point,'tag','fastupdate');

		set(imh,'CloseRequestFcn','showsi_update(''close_attached_scatter'')');		
	else
		
		% it is the dip_image figure

		% update the list of attached figures
		udpim.si_attached_displays=[udpim.si_attached_displays imh];
		
		% display all points already shown in the primary image
		t=udpim.si_points;
		figure(imh);
		for i=1:size(t,1)
			h=line([t(i,1) t(i,1)],[t(i,2),t(i,2)]);
			set(h, 'color', udpim.si_classcolmap(t(i,3),:),'marker','.');
		end

		% add menu items
		h = findobj(imh,'Type','uimenu','tag','actions');
		% visualization mode selector
		uimenu(h,'Label','spectra visualisation','Tag', ...
		       'mouse_si_visualisation','Callback','si_setvismodeatt(gcbf);');
		% put the menu item to the first position:
		h2=get(h,'children');
		set(h,'children',[h2(2:end);h2(1)]);
		si_setvismodeatt(imh);	
		
		% set the callbacks
		set(imh,'WindowButtonMotionFcn','showsi_update(''motion_dip'')');	
		set(imh,'WindowButtonDownFcn','showsi_update(''down_dip'')');
		set(imh,'CloseRequestFcn','showsi_update(''close_attached_display'')');

	end
		
	% store the handles in both plots
	set(sih,'UserData',[]);
	set(sih,'UserData',udpim);

	set(imh,'UserData',[]);
	set(imh,'UserData',udim);	

	