% SIMCAC Supervised SIMCA classifier
%
%   W=SIMCAC(A,M)
%
% DESCRIPTION
% This function implements SIMCA classifier (prtools mapping). It builds a PCA model
% for each class. For a new sample, distance to each model is computed
% and the sample is assigned to the closest class.
%
% SEE ALSO
% SIMCAM

function w=simcac(a,m)

   if (nargin < 1) | (isempty(a) & ~exist('m','var'))  
      % Definition: an untrained mapping.
      w = mapping(mfilename,{});
      w = setname(w,'SIMCA classifier');
     
   elseif isempty(a) & isnumeric(m)
       
      w = mapping(mfilename,{m});
      w = setname(w,'SIMCA classifier');

   elseif ~exist('m','var') | ( exist('m','var') & isnumeric(m))

	Options.use_ecv=1;
	Options.nPC=NaN; % autodetection
  if ~exist('m','var')
	    Options.PEV=0.95;
	else
	    Options.PEV=m;
	end
       
  
  % Training: W will be the trained mapping.
	[m,k,c] = getsize(a);

	[n,lablist]=classsizes(a);

	classdata=simca_train(a,Options);
	
	w=mapping(mfilename,'trained',{classdata},a.lablist,k,c);
      
   else 

      % Execution of the mapping, W will be a dataset.
      v = +m;
      
      classdata=v{1};

      % distance to classes:
      [lab,outl,Q,T2]=simca_apply(a,classdata);      
      d=sqrt(Q.^2 + T2.^2);
      
      % the best is minimum, we turn it by switching the sign:
      w=setdata(a,-d);
      w=setfeatlab(w,getlab(m));
      
   end
   
   return
