% SIMCAM Supervised SIMCA mapping
%
%   W=SIMCAM(A,M)
%
% DESCRIPTION
% This function implements a SIMCA mapping (in prtools sens). For each class in a
% C-class problem a PCA model is built. For new data samples, the trained
% mapping returns a new 2*C dimensional feature space. First C dimensions
% are the distances to class models (Q) and the remaining C dimensions
% represent the distances within the model (T2).
%
% SEE ALSO
% SIMCAC

function w=simcam(a,m)

   if (nargin < 1) | (isempty(a))  
      % Definition: an untrained mapping.
      w = mapping(mfilename,{});
      w = setname(w,'SIMCA mapping');

   elseif ~exist('m','var')
      
      % Training: W will be the trained mapping.
      [m,k,c] = getsize(a);

      [n,lablist]=classsizes(a);

      classdata=simca_train(a);

      w=mapping(mfilename,'trained',{classdata},a.lablist,k,2*c);
      
   else 

      % Execution of the mapping, W will be a dataset.
      v = +m;
      
      classdata=v{1};

      % distance to classes:
      [lab,outl,Q,T2]=simca_apply(a,classdata);      
      
      w=setdat(a,[Q,T2]);
      
   end
   
   return
