% SPECNORMM Spectral normalization mapping
%
%     W = SPECNORMM(A,T)
%
% 'none'       - no normalization
% 'unitarea'   - normalize spectra to unit area (by default)
% 'unitbdl',1  - normalize spectra to unit block-distance length
% 'unitedl',2  - normalize spectra to unit euclidian-distance length

% Copyright: Pavel Paclik, pavel@ph.tn.tudelft.nl
% Faculty of Applied Physics, Delft University of Technology
% P.O. Box 5046, 2600 GA Delft, The Netherlands

function w = specnormm(a,t)

%prtrace(mfilename);
if nargin < 2 | isempty(t)
  t = 'unitarea';
end

if nargin < 1 | isempty(a)
  switch t
	case {'none', 'unitarea', 'unitbdl', 'unitedl'}
    ;  
  otherwise
    if isa(t,'double') 
      if length(t) ~= 1 | t <= 0
        error('the wrong mapping parameter');
      end  
    else  
      error('unknown mapping type');       
    end
  end

  w = mapping(mfilename,'fixed',{t});
  w = setname(w,['Spectral normalization mapping: ' num2str(t)]);
  
else
  disp(t)
	
  switch t
  case 'none'      
    ;  
  case 'unitarea'
    norm=sum(+a,2);
    a = setdata(a,+a./repmat(norm, [1,size(a,2)]));

  otherwise    
    switch t
    case 'unitbdl'
      t = 1;
    case 'unitedl'
      t = 2;
    end
      
    if ~isa(t,'double') | length(t) ~= 1 | t <= 0
      error('the wrong mapping parameter');
    end
    
    if ~isinf(t)
      norm=(sum(abs(+a).^t,2)).^(1/t);
    else
      norm=max(abs(+a),2);
    end
	  
    a = setdata(a,+a./repmat(norm,1,size(a,2)));    
  end    
  
  w=a;  
end
	
return


%       if 0
%       [m, k] = size(a);
%       step = 10000;
%       begs = 1:step:m;
%       ends = begs + (step-1);
%       ends(end) = m;
%       for i=1:length(begs)
%         norm=sum(+a(begs(i):ends(i),:),2);
%         a(begs(i):ends(i),:) = (+a(begs(i):ends(i),:))./repmat(norm, [1,size(a,2)]);
%       end
%       end  
      

