function res = subsample(dset,rc,ss_dset_ind)
%
% Spectral image subsampling 
%
%   ss_dsets = subsample(dset,rc)
%   ss_dset  = subsample(dset,rc,ds_ind)
%
% ss_dset    dataset object containig spectral image
% ss_dsets   cell-array of dataset objects containig subsampled spectral images
%            or dataset (if ds_ind was specified)
%

% Copyright: S.Verzakov, serguei@ph.tn.tudelft.nl
% Faculty of Applied Sciences, Delft University of Technology
% P.O. Box 5046, 2600 GA Delft, The Netherlands

if nargin < 3
  ss_dset_ind = [];
end

objsize = dset.objsize;
rows=objsize(1);
cols=objsize(2);

if isempty(ss_dset_ind)
  res = {};
  for i=1:rc(1)
    rind = [i:rc(1):rows];
    ss_rows = length(rind);

    for j=1:rc(2)
      cind = [j:rc(2):cols];    
      ss_cols = length(cind);    
      [cc, rr] = meshgrid(cind,rind);
      ind = sub2ind(objsize,rr(:),cc(:));
      res{i,j} = dset(ind,:);
      res{i,j}.objsize = [ss_rows ss_cols];
    end
  end

else
  rind = [ss_dset_ind(1):rc(1):rows];
  cind = [ss_dset_ind(2):rc(2):cols];
  ss_rows = length(rind);
  ss_cols = length(cind);    
  [cc, rr] = meshgrid(cind,rind);
  ind = sub2ind(objsize,rr(:),cc(:));
  res = dset(ind,:);
  res.objsize = [ss_rows ss_cols];
end  

return















