function tmr_mnf(h,eventdata,varargin)
% 

% (C) 2006 S. Verzakov, verzakov@tudelft.nl

% title of application
Title = 'MNF/ALS';

% width & font size of input dialog
Width = 0.3;   % normalized
FontSize = 10; % points (9,10,11,12) are recommended

ImmediateApply = -1;

%=== do not change this code (begin)
InpDesc = InputDescription;   
DlgOptions.Title    = Title;
DlgOptions.Width    = Width;
DlgOptions.FontSize = FontSize;
DlgOptions.Mode     = 'Run';
DlgOptions.RunFunc  = @Run;
DlgOptions.FinFunc  = @tm_fin;
DlgOptions.FinInp   = get(h,'Parent');
DlgOptions.ImmApp   = ImmediateApply;
advdlg(InpDesc,DlgOptions)
%=== do not change this code (end)

return;

%------------------------------------------------------- InputDescription ---
% Here we define the parameters that control the appearance of the main (plot) figure (in the left).
% We describe the uicontrols in the input dialog box (right)
% Possible control types are shown and their use is described below

function id = InputDescription

% edit (variable)
id.Dataset.Type = 'variable';
id.Dataset.Prompt = 'Read data from a dataset';
id.Dataset.CanBeEmpty = 1; % default 0
id.Dataset.Mode = 'get'; % default 'put' ('put','get')
id.Dataset.WarnBeforeOverwrite = 0; % default 1 
id.Dataset.MakeGlobal = 0; % default 0 
id.Dataset.Default = 'data'; % default '' 

% delimiter
id.Delimiter1a.Type = 'delimiter';
id.Delimiter1a.Prompt = '';
% default '----------------------------------------------------------------------------------------------------------------------------------------------'

% delimiter
id.Delimiter1b.Type = 'delimiter';
id.Delimiter1b.Prompt = '--- MNF parameters ----------------------------------------';
% default '----------------------------------------------------------------------------------------------------------------------------------------------'

% edit (double)
id.MNF_NumFrac.Type = 'double';
id.MNF_NumFrac.Prompt = 'Number of components/Fraction of explained variance';
id.MNF_NumFrac.Range = [-inf inf];
id.MNF_NumFrac.Inclusive = [1 1];
id.MNF_NumFrac.Length = [1];
id.MNF_NumFrac.Default = [0.95]; %default [] 

% popup 
id.MNF_CritType.Type = 'set';
id.MNF_CritType.Prompt = 'Type of citerion';
id.MNF_CritType.Style = 'pop'; % default 'pop' ('pop', 'box') 
id.MNF_CritType.Return = 'val'; % default 'val' ('val' - return ret val, or string from list if ret val is empty, 'ind' - return index/indices)
id.MNF_CritType.ReturnValue = {'acor', 'acov', 'nf', 'n'}; %cell array of values to be returned, default {}
id.MNF_CritType.List = {'max autocorrelation factors','max autocovriance factors', 'min noise fraction', 'min noise'};
id.MNF_CritType.Default = 1; % default 1

% popup 
id.MNF_EstimMethod.Type = 'set';
id.MNF_EstimMethod.Prompt = 'Estimation method';
id.MNF_EstimMethod.Style = 'pop'; % default 'pop' ('pop', 'box') 
id.MNF_EstimMethod.Return = 'val'; % default 'val' ('val' - return ret val, or string from list if ret val is empty, 'ind' - return index/indices)
id.MNF_EstimMethod.ReturnValue = {'shift', 'smooth'}; %cell array of values to be returned, default {}
id.MNF_EstimMethod.List = {'shift image','smooth image'};
id.MNF_EstimMethod.Default = 1; % default 1


% edit (double)
id.MNF_Param.Type = 'double';
id.MNF_Param.Prompt = 'Size of shift/Sigma of Gaussian';
id.MNF_Param.Range = [0 inf];
id.MNF_Param.Inclusive = [0 0];
id.MNF_Param.Length = [1];
id.MNF_Param.Default = [1]; %default [] 

% checkbox
id.MNF_MeanCentering.Type = 'logical';
id.MNF_MeanCentering.Prompt = 'Center data before factor extraction';
id.MNF_MeanCentering.Default = 1; % default 0

% checkbox
id.MNF_DirtyTrick.Type = 'logical';
id.MNF_DirtyTrick.Prompt = 'Do a dirty trick: compute max autocor as min noise frac';
id.MNF_DirtyTrick.Default = 1; % default 0


% delimiter
id.Delimiter2a.Type = 'delimiter';
id.Delimiter2a.Prompt = '';
% default '----------------------------------------------------------------------------------------------------------------------------------------------'

% delimiter
id.Delimiter2b.Type = 'delimiter';
id.Delimiter2b.Prompt = '--- ALS parameters ------------------------------------------------';
% default '----------------------------------------------------------------------------------------------------------------------------------------------'

% edit (int)
id.ALS_MaxIt.Type = 'int';
id.ALS_MaxIt.Prompt = 'Maximal number of iterations';
id.ALS_MaxIt.Range = [0 inf];
id.ALS_MaxIt.Inclusive = [1 0];
id.ALS_MaxIt.Length = [1];
id.ALS_MaxIt.Default = [0]; %default [] 

% popup 
id.ALS_CritType.Type = 'set';
id.ALS_CritType.Prompt = 'Type of stopping criterion';
id.ALS_CritType.Style = 'pop'; % default 'pop' ('pop', 'box') 
id.ALS_CritType.Return = 'val'; % default 'val' ('val' - return string, 'ind' - return index/indices)
id.ALS_CritType.List = {'convergence','reconstruction'};
id.ALS_CritType.Default = 1; % default 1

% edit (double)
id.ALS_Eps.Type = 'double';
id.ALS_Eps.Prompt = 'Stopping critetion';
id.ALS_Eps.Range = [0 inf];
id.ALS_Eps.Inclusive = [1 0];
id.ALS_Eps.Length = [1];
id.ALS_Eps.Default = [1e-2]; %default [] 

% delimiter
id.Delimiter3a.Type = 'delimiter';
id.Delimiter3a.Prompt = '';
% default '----------------------------------------------------------------------------------------------------------------------------------------------'

% delimiter
id.Delimiter3b.Type = 'delimiter';
id.Delimiter3b.Prompt = '--- Output ---------------------------------------------------------------';
% default '----------------------------------------------------------------------------------------------------------------------------------------------'

% edit (double)
id.ProgRep.Type = 'double';
id.ProgRep.Prompt = 'Progress report file handle (1 means command window)';
id.ProgRep.Length = [1];
id.ProgRep.Default = [1]; % default [] 
id.ProgRep.CanBeEmpty = 1;

% edit (variable)
id.Out.Type = 'variable';
id.Out.Prompt = 'Save data in a structure';
id.Out.CanBeEmpty = 1; % default 0
id.Out.Mode = 'put'; % default 'put' ('put','get')
id.Out.WarnBeforeOverwrite = 0; % default 1 
id.Out.MakeGlobal = 0; % default 0 
id.Out.Default = 'out'; % default '' 
id.Out.Enable = 'on'; % default 'on'

return;


%------------------------------------------------------- Run ---
% It is the response to the 'Run' button in the input dialog.
% We take the parameter values from  the input dialog (the right one) and, with them, 
% we run the algorithm and plot the results in the plot (left) window

function Run(in)

layout = [2 2];

h(1) = tm_figure(['MNF extracted channels'], 'MNF_ExtChan', 'close', [layout 1]);  clf;
h(2) = tm_figure('MNF error', 'MNF_Err', 'close', [layout 2]);  clf;

if in.ALS_MaxIt > 0
  h(3) = tm_figure('Pure spectra (refined by ALS)',   'MNF_Spec',     'close', [layout 3]);  clf;
  h(4) = tm_figure('Pure channels (refined by ALS)',  'MNF_Chan',     'close', [layout 4]);  clf;
else
  h(3) = tm_figure('MNF: Spectra',   'MNF_Spec',     'close', [layout 3]);  clf;
  h(4) = tm_figure('MNF: Channels',  'MNF_Chan',     'close', [layout 4]);  clf;
end  

drawnow

x = tm_getvar(in.Dataset);

opt_als.max_iter = in.ALS_MaxIt;
opt_als.crit     = in.ALS_CritType;
opt_als.eps      = in.ALS_Eps;
opt_als.verbose  = in.ProgRep;

[out.w, num_frac,crit,frac,tot] = mnfm(x, in.MNF_NumFrac, in.MNF_CritType, in.MNF_EstimMethod, in.MNF_Param, in.MNF_MeanCentering, in.MNF_DirtyTrick);
out.k_ext = size(out.w,2);
out.frac = frac;
out.crit = crit;

w = out.w;
wdata = +w;
offset = wdata.offset;
wdata.offset = zeros(size(offset));
w.data = wdata;
out.z0 = remove_spectra_info(x*w);
out.z0.name = [x.name ': extracted channels'];
%out.z0 = remove_spectra_info(x*out.w);

figure(h(1));
showspim(out.z0)
daspect([1 1 1])
colormap(jet)
shg
  
figure(h(2))
subplot(2,1,1)
plot(1-cumsum(out.frac(1:min(out.k_ext+3,end))));
legend('relative squared reconstruction error');


subplot(2,1,1)
plot(1-cumsum(out.frac(1:min(out.k_ext+3,end))));
legend('relative squared reconstruction error');

switch in.MNF_CritType
case  'nf'
  subplot(2,1,2)
  plot(crit(1:min(out.k_ext+3,end)))
  if in.MNF_MeanCentering   
    legend('noise var/component variance')
  else
    legend('noise MSS/component MSS')
  end
  
case  'n'
  subplot(2,1,2)
  plot(crit(1:min(out.k_ext+3,end))/tot);
  if in.MNF_MeanCentering   
    legend('noise var/total variance')
  else
    legend('noise MSS/total MSS')
  end
    
case  'acor'
  subplot(2,1,2)
  plot(crit(1:min(out.k_ext+3,end)))
  if in.MNF_MeanCentering   
    legend('autocorrelation')
  else
    legend('uncentered autcorrelation')
  end
    
case  'acov'
  subplot(2,1,1)
  plot([1-cumsum(out.frac(1:min(out.k_ext+3,end))), 1-cumsum(crit(1:min(out.k_ext+3,end)))/sum(crit)]);
  legend('relative squared reconstruction error', 'unexplained autocovariance fraction');
  
  subplot(2,1,2)
  plot(crit(1:min(out.k_ext+3,end))/tot)
  if in.MNF_MeanCentering   
    legend('autocovariance/total variance')
  else
    legend('uncentered autocovariance/total MSS')
  end
end
    
xlabel('# of components')
drawnow

if opt_als.max_iter > 0
  opt_als.mode     = 'features';
  [out.z, out.y] = als(x,out.z0,opt_als);
else
  out.z = out.z0;
  out.y = (+out.z)\(+x);
  m = size(x,1);

  out.y = dataset(out.y);
  out.y.featlab  = x.featlab;
  out.y.featsize = x.featsize;
  out.y.user     = x.user;
  out.y.name = [x.name ': extracted spectra'];
end  

figure(h(3))
showspim(out.y)
shg
  
figure(h(4));
showspim(out.z)
colormap(jet)
shg
  
drawnow

tm_putvar(in.Out, out);

return

