function tmr_opa(h,eventdata,varargin)
% 

% (C) 2006 S. Verzakov, verzakov@tudelft.nl

% title of application
Title = 'OPA/ALS';

% width & font size of input dialog
Width = 0.3;   % normalized
FontSize = 10; % points (9,10,11,12) are recommended

ImmediateApply = -1;

%=== do not change this code (begin)
InpDesc = InputDescription;   
DlgOptions.Title    = Title;
DlgOptions.Width    = Width;
DlgOptions.FontSize = FontSize;
DlgOptions.Mode     = 'Run';
DlgOptions.RunFunc  = @Run;
DlgOptions.FinFunc  = @tm_fin;
DlgOptions.FinInp   = get(h,'Parent');
DlgOptions.ImmApp   = ImmediateApply;
advdlg(InpDesc,DlgOptions)
%=== do not change this code (end)

return;

%------------------------------------------------------- InputDescription ---
% Here we define the parameters that control the appearance of the main (plot) figure (in the left).
% We describe the uicontrols in the input dialog box (right)
% Possible control types are shown and their use is described below

function id = InputDescription

% edit (variable)
id.Dataset.Type = 'variable';
id.Dataset.Prompt = 'Read data from a dataset';
id.Dataset.CanBeEmpty = 1; % default 0
id.Dataset.Mode = 'get'; % default 'put' ('put','get')
id.Dataset.WarnBeforeOverwrite = 0; % default 1 
id.Dataset.MakeGlobal = 0; % default 0 
id.Dataset.Default = 'data'; % default '' 

% delimiter
id.Delimiter1a.Type = 'delimiter';
id.Delimiter1a.Prompt = '';
% default '----------------------------------------------------------------------------------------------------------------------------------------------'

% delimiter
id.Delimiter1b.Type = 'delimiter';
id.Delimiter1b.Prompt = '--- OPA parameters ------------------------------------------------';
% default '----------------------------------------------------------------------------------------------------------------------------------------------'

% popup 
id.OPA_Mode.Type = 'set';
id.OPA_Mode.Prompt = 'Mode';
id.OPA_Mode.Style = 'pop'; % default 'pop' ('pop', 'box') 
id.OPA_Mode.Return = 'val'; % default 'val' ('val' - return string, 'ind' - return index/indices)
id.OPA_Mode.List = {'pure spectra','pure channels'};
id.OPA_Mode.Default = 1; % default 1

% edit (int)
id.OPA_MaxNum.Type = 'int';
id.OPA_MaxNum.Prompt = 'Maximal number of components [1, inf]';
id.OPA_MaxNum.Range = [1 inf];
id.OPA_MaxNum.Inclusive = [1 1];
id.OPA_MaxNum.Length = [1];
id.OPA_MaxNum.Default = [inf]; %default [] 

% popup 
%id.OPA_IndepMeas.Type = 'set';
%id.OPA_IndepMeas.Prompt = 'Independance measure (inluences returned dissim values only)';
%id.OPA_IndepMeas.Style = 'pop'; % default 'pop' ('pop', 'box') 
%id.OPA_IndepMeas.Return = 'val'; % default 'val' ('val' - return string, 'ind' - return index/indices)
%id.OPA_IndepMeas.List = {'determinant','projection'};
%id.OPA_IndepMeas.Default = 1; % default 1

% edit (double)
id.OPA_Eps.Type = 'double';
id.OPA_Eps.Prompt = 'Stopping critetion';
id.OPA_Eps.Range = [0 inf];
id.OPA_Eps.Inclusive = [1 0];
id.OPA_Eps.Length = [1];
id.OPA_Eps.Default = [1e-2]; %default [] 

% delimiter
id.Delimiter2a.Type = 'delimiter';
id.Delimiter2a.Prompt = '';
% default '----------------------------------------------------------------------------------------------------------------------------------------------'

% delimiter
id.Delimiter2b.Type = 'delimiter';
id.Delimiter2b.Prompt = '--- ALS parameters ------------------------------------------------';
% default '----------------------------------------------------------------------------------------------------------------------------------------------'

% edit (int)
id.ALS_MaxIt.Type = 'int';
id.ALS_MaxIt.Prompt = 'Maximal number of iterations';
id.ALS_MaxIt.Range = [0 inf];
id.ALS_MaxIt.Inclusive = [1 0];
id.ALS_MaxIt.Length = [1];
id.ALS_MaxIt.Default = [0]; %default [] 

% popup 
id.ALS_CritType.Type = 'set';
id.ALS_CritType.Prompt = 'Type of stopping criterion';
id.ALS_CritType.Style = 'pop'; % default 'pop' ('pop', 'box') 
id.ALS_CritType.Return = 'val'; % default 'val' ('val' - return string, 'ind' - return index/indices)
id.ALS_CritType.List = {'convergence','reconstruction'};
id.ALS_CritType.Default = 1; % default 1

% edit (double)
id.ALS_Eps.Type = 'double';
id.ALS_Eps.Prompt = 'Stopping critetion';
id.ALS_Eps.Range = [0 inf];
id.ALS_Eps.Inclusive = [1 0];
id.ALS_Eps.Length = [1];
id.ALS_Eps.Default = [1e-2]; %default [] 

% delimiter
id.Delimiter3a.Type = 'delimiter';
id.Delimiter3a.Prompt = '';
% default '----------------------------------------------------------------------------------------------------------------------------------------------'

% delimiter
id.Delimiter3b.Type = 'delimiter';
id.Delimiter3b.Prompt = '--- Output ---------------------------------------------------------------';
% default '----------------------------------------------------------------------------------------------------------------------------------------------'

% edit (double)
id.ProgRep.Type = 'double';
id.ProgRep.Prompt = 'Progress report file handle (1 means command window)';
id.ProgRep.Length = [1];
id.ProgRep.Default = [1]; % default [] 
id.ProgRep.CanBeEmpty = 1;

% edit (variable)
id.Out.Type = 'variable';
id.Out.Prompt = 'Save data in a structure';
id.Out.CanBeEmpty = 1; % default 0
id.Out.Mode = 'put'; % default 'put' ('put','get')
id.Out.WarnBeforeOverwrite = 0; % default 1 
id.Out.MakeGlobal = 0; % default 0 
id.Out.Default = 'out'; % default '' 
id.Out.Enable = 'on'; % default 'on'

return;


%------------------------------------------------------- Run ---
% It is the response to the 'Run' button in the input dialog.
% We take the parameter values from  the input dialog (the right one) and, with them, 
% we run the algorithm and plot the results in the plot (left) window

function Run(in)

layout = [2 2];

h(1) = tm_figure(['OPA ' in.OPA_Mode ' selection'], 'OPA_SpecChan', 'close', [layout 1]);  clf;
h(2) = tm_figure('OPA dissim and error',            'OPA_Diss&Err', 'close', [layout 2]);  clf;

if in.ALS_MaxIt > 0
  h(3) = tm_figure('Pure spectra (refined by ALS)',   'OPA_Spec',     'close', [layout 3]);  clf;
  h(4) = tm_figure('Pure channels (refined by ALS)',  'OPA_Chan',     'close', [layout 4]);  clf;
else
  h(3) = tm_figure('OPA: Spectra',   'OPA_Spec',     'close', [layout 3]);  clf;
  h(4) = tm_figure('OPA: Channels',  'OPA_Chan',     'close', [layout 4]);  clf;
end  


drawnow

x = tm_getvar(in.Dataset);
  
opt_opa.max_num = in.OPA_MaxNum;

%switch in.OPA_IndepMeas
%case 'absolute'
%  opt_opa.indep_meas = 'abs';  
%case 'relaive'
%  opt_opa.indep_meas = 'rel';    
%end

opt_opa.eps = in.OPA_Eps;
opt_opa.verbose = in.ProgRep;

opt_als.max_iter = in.ALS_MaxIt;
opt_als.crit     = in.ALS_CritType;
opt_als.eps      = in.ALS_Eps;
opt_als.verbose  = in.ProgRep;
%opt_als.crop_mode = 'features';

switch in.OPA_Mode
case 'pure spectra'
  [out.y0, out.sel_obj_ind, out.sel_obj_dis, out.dis, out.err] = opaps(x,opt_opa);

  
  figure(h(1))
  [rind, cind] = ind2sub(x.objsize,out.sel_obj_ind);
  ha = showspim(out.y0,[],[rind(:), cind(:)]);
  shg
  
  figure(h(2))
  subplot(3,1,1)
  plot(out.sel_obj_dis(:,1))
  ylabel('abs dissim')
  subplot(3,1,2)
  plot(out.sel_obj_dis(:,2))
  ylabel('rel dissim')
  subplot(3,1,3)
  plot(out.err)
  ylabel('rel sqr proj err')
  xlabel('# of components')
  shg
  
  drawnow
  
  if 0
  ind = [1,4,6,10,11,14,15];
  xind = 1:length(out.sel_obj_ind);
  xind(ind) = [];
  out.out0 = out;
  out.y0(xind,:) = [];
  out.sel_obj_ind(xind) = [];
  out.sel_obj_dis(xind,:) = []; 
  out.dis(:,xind,:) = []; 
  out.err(xind) = [];
  [rind, cind] = ind2sub(x.objsize,out.sel_obj_ind);
  end

  
  if opt_als.max_iter > 0
    opt_als.mode = 'samples';
    [out.z, out.y] = als(x,out.y0,opt_als);
    y_desc = [repmat('spec ',[size(out.y0,1) 1]) num2str((1:size(out.y0,1))') repmat(': r=',[size(out.y0,1) 1]) num2str(rind(:)) repmat(', c=',[size(out.y0,1) 1]) num2str(cind(:)) repmat(', refined',[size(out.y0,1) 1])];
  else
    out.y = out.y0;  
    out.z = remove_spectra_info(x);
    out.z = setdata(out.z,(+x)/(+out.y),(1:size(out.y,1))');  
    out.z.name = [out.z.name ': extracted channels'];
  end  


case 'pure channels'
  [out.w, out.sel_var_dis, out.dis, out.err] = opam(x,opt_opa);
  out.z0 = x*out.w;
  
  figure(h(1));
  ha = showspim(out.z0);
  daspect([1 1 1])
  colormap(jet)
  shg
  
  figure(h(2))
  subplot(3,1,1)
  plot(out.sel_var_dis(:,1))
  ylabel('abs dissim')
  subplot(3,1,2)
  plot(out.sel_var_dis(:,2))
  ylabel('rel dissim')
  subplot(3,1,3)
  plot(out.err)
  ylabel('rel squared proj err')
  xlabel('# of components')
  shg

  drawnow
  
  if opt_als.max_iter > 0
    opt_als.mode = 'features';
    [out.z, out.y] = als(x,out.z0,opt_als);
    
    units = specunits(x);
    if ~isempty(units)
      units = [' ' units];  
    end
    out.z.featlab = [num2str(out.z0.featlab) repmat([units ', refined'],[size(out.z0,2) 1])];
  else
    out.z = out.z0;
    out.y = (+out.z)\(+x);
    out.y = dataset(out.y);
    out.y.featlab  = x.featlab;
    out.y.featsize = x.featsize;
    out.y.user     = x.user;
    out.y.name     = [x.name ': extracted spectra']
  end  
end

figure(h(3))
if isfield(out,'y0');
  if opt_als.max_iter == 0 
    showspim(out.y,[],[rind(:),cind(:)])
  else
    showspim(out.y,[],y_desc)
  end
else
  showspim(out.y,[],[],out.w.data.ind);
end
shg
  
figure(h(4));
if isfield(out,'y0');
  showspim(out.z,[],[],[rind(:),cind(:)])
else
  showspim(out.z);
end
shg
  
drawnow

tm_putvar(in.Out, out);

return

