function tmr_simplisma(h,eventdata,varargin)
% 

% (C) 2006 S. Verzakov, verzakov@tudelft.nl

% title of application
Title = 'SIMPLISMA/ALS';

% width & font size of input dialog
Width = 0.3;   % normalized
FontSize = 10; % points (9,10,11,12) are recommended

ImmediateApply = -1;

%=== do not change this code (begin)
InpDesc = InputDescription;   
DlgOptions.Title    = Title;
DlgOptions.Width    = Width;
DlgOptions.FontSize = FontSize;
DlgOptions.Mode     = 'Run';
DlgOptions.RunFunc  = @Run;
DlgOptions.FinFunc  = @tm_fin;
DlgOptions.FinInp   = get(h,'Parent');
DlgOptions.ImmApp   = ImmediateApply;
advdlg(InpDesc,DlgOptions)
%=== do not change this code (end)

return;

%------------------------------------------------------- InputDescription ---
% Here we define the parameters that control the appearance of the main (plot) figure (in the left).
% We describe the uicontrols in the input dialog box (right)
% Possible control types are shown and their use is described below

function id = InputDescription

% edit (variable)
id.Dataset.Type = 'variable';
id.Dataset.Prompt = 'Read data from a dataset';
id.Dataset.CanBeEmpty = 1; % default 0
id.Dataset.Mode = 'get'; % default 'put' ('put','get')
id.Dataset.WarnBeforeOverwrite = 0; % default 1 
id.Dataset.MakeGlobal = 0; % default 0 
id.Dataset.Default = 'data'; % default '' 

% delimiter
id.Delimiter1a.Type = 'delimiter';
id.Delimiter1a.Prompt = '';
% default '----------------------------------------------------------------------------------------------------------------------------------------------'

% delimiter
id.Delimiter1b.Type = 'delimiter';
id.Delimiter1b.Prompt = '--- SIMPLISMA parameters ----------------------------------------';
% default '----------------------------------------------------------------------------------------------------------------------------------------------'

% popup 
id.SIMPLISMA_Mode.Type = 'set';
id.SIMPLISMA_Mode.Prompt = 'Mode';
id.SIMPLISMA_Mode.Style = 'pop'; % default 'pop' ('pop', 'box') 
id.SIMPLISMA_Mode.Return = 'val'; % default 'val' ('val' - return string, 'ind' - return index/indices)
id.SIMPLISMA_Mode.List = {'pure spectra','pure channels'};
id.SIMPLISMA_Mode.Default = 1; % default 1

% edit (int)
id.SIMPLISMA_MaxNum.Type = 'int';
id.SIMPLISMA_MaxNum.Prompt = 'Maximal number of components [1, inf]';
id.SIMPLISMA_MaxNum.Range = [1 inf];
id.SIMPLISMA_MaxNum.Inclusive = [1 1];
id.SIMPLISMA_MaxNum.Length = [1];
id.SIMPLISMA_MaxNum.Default = [inf]; %default [] 

% popup 
%id.SIMPLISMA_IndepMeas.Type = 'set';
%id.SIMPLISMA_IndepMeas.Prompt = 'Independance measure (inluences returned dissim values only)';
%id.SIMPLISMA_IndepMeas.Style = 'pop'; % default 'pop' ('pop', 'box') 
%id.SIMPLISMA_IndepMeas.Return = 'val'; % default 'val' ('val' - return string, 'ind' - return index/indices)
%id.SIMPLISMA_IndepMeas.List = {'determinant','projection'};
%id.SIMPLISMA_IndepMeas.Default = 1; % default 1

% edit (double)
id.SIMPLISMA_Theta.Type = 'double';
id.SIMPLISMA_Theta.Prompt = 'Theta: regularization offset';
id.SIMPLISMA_Theta.Range = [0 inf];
id.SIMPLISMA_Theta.Inclusive = [1 0];
id.SIMPLISMA_Theta.Length = [1];
id.SIMPLISMA_Theta.Default = [1e-2]; %default [] 

% edit (double)
id.SIMPLISMA_Eps.Type = 'double';
id.SIMPLISMA_Eps.Prompt = 'Stopping critetion';
id.SIMPLISMA_Eps.Range = [0 inf];
id.SIMPLISMA_Eps.Inclusive = [1 0];
id.SIMPLISMA_Eps.Length = [1];
id.SIMPLISMA_Eps.Default = [1e-2]; %default [] 

% delimiter
id.Delimiter2a.Type = 'delimiter';
id.Delimiter2a.Prompt = '';
% default '----------------------------------------------------------------------------------------------------------------------------------------------'

% delimiter
id.Delimiter2b.Type = 'delimiter';
id.Delimiter2b.Prompt = '--- ALS parameters ------------------------------------------------';
% default '----------------------------------------------------------------------------------------------------------------------------------------------'

% edit (int)
id.ALS_MaxIt.Type = 'int';
id.ALS_MaxIt.Prompt = 'Maximal number of iterations';
id.ALS_MaxIt.Range = [0 inf];
id.ALS_MaxIt.Inclusive = [1 0];
id.ALS_MaxIt.Length = [1];
id.ALS_MaxIt.Default = [0]; %default [] 

% popup 
id.ALS_CritType.Type = 'set';
id.ALS_CritType.Prompt = 'Type of stopping criterion';
id.ALS_CritType.Style = 'pop'; % default 'pop' ('pop', 'box') 
id.ALS_CritType.Return = 'val'; % default 'val' ('val' - return string, 'ind' - return index/indices)
id.ALS_CritType.List = {'convergence','reconstruction'};
id.ALS_CritType.Default = 1; % default 1

% edit (double)
id.ALS_Eps.Type = 'double';
id.ALS_Eps.Prompt = 'Stopping critetion';
id.ALS_Eps.Range = [0 inf];
id.ALS_Eps.Inclusive = [1 0];
id.ALS_Eps.Length = [1];
id.ALS_Eps.Default = [1e-2]; %default [] 

% delimiter
id.Delimiter3a.Type = 'delimiter';
id.Delimiter3a.Prompt = '';
% default '----------------------------------------------------------------------------------------------------------------------------------------------'

% delimiter
id.Delimiter3b.Type = 'delimiter';
id.Delimiter3b.Prompt = '--- Output ---------------------------------------------------------------';
% default '----------------------------------------------------------------------------------------------------------------------------------------------'

% edit (double)
id.ProgRep.Type = 'double';
id.ProgRep.Prompt = 'Progress report file handle (1 means command window)';
id.ProgRep.Length = [1];
id.ProgRep.Default = [1]; % default [] 
id.ProgRep.CanBeEmpty = 1;

% edit (variable)
id.Out.Type = 'variable';
id.Out.Prompt = 'Save data in a structure';
id.Out.CanBeEmpty = 1; % default 0
id.Out.Mode = 'put'; % default 'put' ('put','get')
id.Out.WarnBeforeOverwrite = 0; % default 1 
id.Out.MakeGlobal = 0; % default 0 
id.Out.Default = 'out'; % default '' 
id.Out.Enable = 'on'; % default 'on'

return;


%------------------------------------------------------- Run ---
% It is the response to the 'Run' button in the input dialog.
% We take the parameter values from  the input dialog (the right one) and, with them, 
% we run the algorithm and plot the results in the plot (left) window

function Run(in)

layout = [2 2];

h(1) = tm_figure(['SIMPLISMA ' in.SIMPLISMA_Mode ' selection'], 'SIMPLISMA_SpecChan', 'close', [layout 1]);  clf;
h(2) = tm_figure('SIMPLISMA purity and error',            'SIMPLISMA_Pur&Err', 'close', [layout 2]);  clf;

if in.ALS_MaxIt > 0
  h(3) = tm_figure('Pure spectra (refined by ALS)',   'SIMPLISMA_Spec',     'close', [layout 3]);  clf;
  h(4) = tm_figure('Pure channels (refined by ALS)',  'SIMPLISMA_Chan',     'close', [layout 4]);  clf;
else
  h(3) = tm_figure('SIMPLISMA: Spectra',   'SIMPLISMA_Spec',     'close', [layout 3]);  clf;
  h(4) = tm_figure('SIMPLISMA: Channels',  'SIMPLISMA_Chan',     'close', [layout 4]);  clf;
end  


drawnow

x = tm_getvar(in.Dataset);

opt_simplisma.max_num = in.SIMPLISMA_MaxNum;
opt_simplisma.theta   = in.SIMPLISMA_Theta;
opt_simplisma.eps     = in.SIMPLISMA_Eps;
opt_simplisma.verbose = in.ProgRep;

opt_als.max_iter = in.ALS_MaxIt;
opt_als.crit     = in.ALS_CritType;
opt_als.eps      = in.ALS_Eps;
opt_als.verbose  = in.ProgRep;


switch in.SIMPLISMA_Mode
case 'pure spectra'
  [out.y0, out.sel_obj_ind, out.sel_obj_purity, out.purity, out.R_ratio, out.err] = simplismaps(x, opt_simplisma);
  
  figure(h(1))
  [rind, cind] = ind2sub(x.objsize,out.sel_obj_ind);
  showspim(out.y0,[],[rind(:) cind(:)])
  shg
  
  figure(h(2))
  subplot(3,1,1)
  plot(out.sel_obj_purity(:,1))
  ylabel('abs purity')
  subplot(3,1,2)
  plot(out.sel_obj_purity(:,2))
  ylabel('rel purity')
  subplot(3,1,3)
  plot([1:length(out.err)]', [out.R_ratio, out.err])
  ylabel('R_{ratio} and err')
  xlabel('# of components')
  shg
  
  drawnow

  if opt_als.max_iter > 0
    opt_als.mode = 'samples';
    [out.z, out.y] = als(x,out.y0,opt_als);
    y_desc = [repmat('spec ',[size(out.y0,1) 1]) num2str((1:size(out.y0))') repmat(': r=',[size(out.y0,1) 1]) num2str(rind(:)) repmat(', c=',[size(out.y0,1) 1]) num2str(cind(:)) repmat(', refined',[size(out.y0,1) 1])];
  else
    out.y = out.y0;  
    out.z = remove_spectra_info(x);
    out.z = setdata(out.z,(+x)/(+out.y),(1:size(out.y,1))');  
    out.z.name = [out.z.name ': extracted channels'];
  end  


case 'pure channels'
  [out.w, out.sel_var_purity, out.purity, out.R_ratio, out.err] = simplismam(x, opt_simplisma);
  out.z0 = x*out.w;

  figure(h(1));
  showspim(out.z0)
  daspect([1 1 1])
  colormap(jet)
  shg
  
  figure(h(2))
  subplot(3,1,1)
  plot(out.sel_var_purity(:,1))
  ylabel('abs purity')
  subplot(3,1,2)
  plot(out.sel_var_purity(:,2))
  ylabel('rel purity')
  subplot(3,1,3)
  plot([1:length(out.err)]', [out.R_ratio, out.err])
  ylabel('R_{ratio} and err')
  xlabel('# of components')
  shg
  
  drawnow
  
  if opt_als.max_iter > 0
    opt_als.mode = 'features';    
    [out.z, out.y] = als(x,out.z0,opt_als);
    
    units = specunits(x);
    if ~isempty(units)
      units = [' ' units];  
    end
    out.z.featlab = [num2str(out.z0.featlab) repmat([units ', refined'],[size(out.z0,2) 1])];

  else
    out.z = out.z0;
    out.y = (+out.z)\(+x);
    out.y = dataset(out.y);
    out.y.featlab  = x.featlab;
    out.y.featsize = x.featsize;
    out.y.user     = x.user;
    out.y.name     = [x.name ': extracted spectra']
  end  
end


figure(h(3))
if isfield(out,'y0');
  if opt_als.max_iter == 0 
    showspim(out.y,[],[rind(:),cind(:)])
  else
    showspim(out.y,[],y_desc)
  end
else
  showspim(out.y,[],[],out.w.data.ind);
end
shg
  
figure(h(4));
if isfield(out,'y0');
  showspim(out.z,[],[],[rind(:),cind(:)])
else
  showspim(out.z);
end
shg
  
drawnow

tm_putvar(in.Out, out);

return

