function tmr_varimax(h,eventdata,varargin)
% 

% (C) 2006 S. Verzakov, verzakov@tudelft.nl

% title of application
Title = 'PCA/VARIMAX/ALS';

% width & font size of input dialog
Width = 0.3;   % normalized
FontSize = 10; % points (9,10,11,12) are recommended

ImmediateApply = -1;

%=== do not change this code (begin)
InpDesc = InputDescription;   
DlgOptions.Title    = Title;
DlgOptions.Width    = Width;
DlgOptions.FontSize = FontSize;
DlgOptions.Mode     = 'Run';
DlgOptions.RunFunc  = @Run;
DlgOptions.FinFunc  = @tm_fin;
DlgOptions.FinInp   = get(h,'Parent');
DlgOptions.ImmApp   = ImmediateApply;
advdlg(InpDesc,DlgOptions)
%=== do not change this code (end)

return;

%------------------------------------------------------- InputDescription ---
% Here we define the parameters that control the appearance of the main (plot) figure (in the left).
% We describe the uicontrols in the input dialog box (right)
% Possible control types are shown and their use is described below

function id = InputDescription

% edit (variable)
id.Dataset.Type = 'variable';
id.Dataset.Prompt = 'Read data from a dataset';
id.Dataset.CanBeEmpty = 1; % default 0
id.Dataset.Mode = 'get'; % default 'put' ('put','get')
id.Dataset.WarnBeforeOverwrite = 0; % default 1 
id.Dataset.MakeGlobal = 0; % default 0 
id.Dataset.Default = 'data'; % default '' 

% delimiter
id.Delimiter1a.Type = 'delimiter';
id.Delimiter1a.Prompt = '';
% default '----------------------------------------------------------------------------------------------------------------------------------------------'

% delimiter
id.Delimiter1b.Type = 'delimiter';
id.Delimiter1b.Prompt = '--- PCA/VARIMAX parameters ----------------------------------------';
% default '----------------------------------------------------------------------------------------------------------------------------------------------'

% popup 
id.VARIMAX_Mode.Type = 'set';
id.VARIMAX_Mode.Prompt = 'Mode';
id.VARIMAX_Mode.Style = 'pop'; % default 'pop' ('pop', 'box') 
id.VARIMAX_Mode.Return = 'val'; % default 'val' ('val' - return string, 'ind' - return index/indices)
id.VARIMAX_Mode.List = {'sparse spectra (pixels)','sparse channels (images)'};
id.VARIMAX_Mode.Default = 1; % default 1

% edit (double)
id.VARIMAX_NumFrac.Type = 'double';
id.VARIMAX_NumFrac.Prompt = 'Number of components/Fraction of explained sum of squares';
id.VARIMAX_NumFrac.Range = [0 inf];
id.VARIMAX_NumFrac.Inclusive = [0 0];
id.VARIMAX_NumFrac.Length = [1];
id.VARIMAX_NumFrac.Default = [0.95]; %default [] 

% edit (int)
id.VARIMAX_NAttempts.Type = 'int';
id.VARIMAX_NAttempts.Prompt = 'Number of attempts';
id.VARIMAX_NAttempts.Range = [1 inf];
id.VARIMAX_NAttempts.Inclusive = [1 0];
id.VARIMAX_NAttempts.Length = [1];
id.VARIMAX_NAttempts.Default = [3]; %default [] 

% edit (int)
id.VARIMAX_MaxIter.Type = 'int';
id.VARIMAX_MaxIter.Prompt = 'Number of full iterations within an attempt';
id.VARIMAX_MaxIter.Range = [0 inf];
id.VARIMAX_MaxIter.Inclusive = [1 0];
id.VARIMAX_MaxIter.Length = [1];
id.VARIMAX_MaxIter.Default = [3]; %default [] 

% edit (double)
id.VARIMAX_Eps.Type = 'double';
id.VARIMAX_Eps.Prompt = 'Stopping critetion: (new_var-old_var) < eps*old_var';
id.VARIMAX_Eps.Range = [0 1];
id.VARIMAX_Eps.Inclusive = [0 0];
id.VARIMAX_Eps.Length = [1];
id.VARIMAX_Eps.Default = [1e-6]; %default [] 

% delimiter
id.Delimiter2a.Type = 'delimiter';
id.Delimiter2a.Prompt = '';
% default '----------------------------------------------------------------------------------------------------------------------------------------------'

% delimiter
id.Delimiter2b.Type = 'delimiter';
id.Delimiter2b.Prompt = '--- ALS parameters ------------------------------------------------';
% default '----------------------------------------------------------------------------------------------------------------------------------------------'

% edit (int)
id.ALS_MaxIt.Type = 'int';
id.ALS_MaxIt.Prompt = 'Maximal number of iterations';
id.ALS_MaxIt.Range = [0 inf];
id.ALS_MaxIt.Inclusive = [1 0];
id.ALS_MaxIt.Length = [1];
id.ALS_MaxIt.Default = [0]; %default [] 

% popup 
id.ALS_CritType.Type = 'set';
id.ALS_CritType.Prompt = 'Type of stopping criterion';
id.ALS_CritType.Style = 'pop'; % default 'pop' ('pop', 'box') 
id.ALS_CritType.Return = 'val'; % default 'val' ('val' - return string, 'ind' - return index/indices)
id.ALS_CritType.List = {'convergence','reconstruction'};
id.ALS_CritType.Default = 1; % default 1

% edit (double)
id.ALS_Eps.Type = 'double';
id.ALS_Eps.Prompt = 'Stopping critetion';
id.ALS_Eps.Range = [0 inf];
id.ALS_Eps.Inclusive = [1 0];
id.ALS_Eps.Length = [1];
id.ALS_Eps.Default = [1e-2]; %default [] 

% delimiter
id.Delimiter3a.Type = 'delimiter';
id.Delimiter3a.Prompt = '';
% default '----------------------------------------------------------------------------------------------------------------------------------------------'

% delimiter
id.Delimiter3b.Type = 'delimiter';
id.Delimiter3b.Prompt = '--- Output ---------------------------------------------------------------';
% default '----------------------------------------------------------------------------------------------------------------------------------------------'

% edit (double)
id.ProgRep.Type = 'double';
id.ProgRep.Prompt = 'Progress report file handle (1 means command window)';
id.ProgRep.Length = [1];
id.ProgRep.Default = [1]; % default [] 
id.ProgRep.CanBeEmpty = 1;

% edit (variable)
id.Out.Type = 'variable';
id.Out.Prompt = 'Save data in a structure';
id.Out.CanBeEmpty = 1; % default 0
id.Out.Mode = 'put'; % default 'put' ('put','get')
id.Out.WarnBeforeOverwrite = 0; % default 1 
id.Out.MakeGlobal = 0; % default 0 
id.Out.Default = 'out'; % default '' 
id.Out.Enable = 'on'; % default 'on'

return;


%------------------------------------------------------- Run ---
% It is the response to the 'Run' button in the input dialog.
% We take the parameter values from  the input dialog (the right one) and, with them, 
% we run the algorithm and plot the results in the plot (left) window

function Run(in)

layout = [2 2];

h(1) = tm_figure(['VARIMAX ' in.VARIMAX_Mode ' extraction'], 'VARIMAX_SpecChan', 'close', [layout 1]);  clf;
h(2) = tm_figure('PCA err and VARIMAX sparsity', 'VARIMAX_Err&Spars', 'close', [layout 2]);  clf;

if in.ALS_MaxIt > 0
  h(3) = tm_figure('Pure spectra (refined by ALS)',   'VARIMAX_Spec', 'close', [layout 3]);  clf;
  h(4) = tm_figure('Pure channels (refined by ALS)',  'VARIMAX_Chan', 'close', [layout 4]);  clf;
else
  h(3) = tm_figure('VARIMAX: Sectra',   'VARIMAX_Spec',  'close', [layout 3]);  clf;
  h(4) = tm_figure('VARIMAX: Channels', 'VARIMAX_Chan',  'close', [layout 4]);  clf;
end


drawnow

x = tm_getvar(in.Dataset);
  
opt_varimax.n_attempts = in.VARIMAX_NAttempts;
opt_varimax.max_iter   = in.VARIMAX_MaxIter;
opt_varimax.eps        = in.VARIMAX_Eps;
opt_varimax.verbose    = in.ProgRep;

opt_als.max_iter = in.ALS_MaxIt;
opt_als.crit     = in.ALS_CritType;
opt_als.eps      = in.ALS_Eps;
opt_als.verbose  = in.ProgRep;


switch in.VARIMAX_Mode
case 'sparse spectra (pixels)'
  [out.w, out.k_ext, out.frac, out.v, out.z, out.y0] = varimaxom(x, in.VARIMAX_NumFrac, opt_varimax);

  %out.y0 = dataset((out.w.data.rot)');
  %out.y0.featlab = x.featlab;
  %out.y0.featsize = x.featsize;
  %out.y0.user = x.user;
  
  figure(h(1))
  showspim(out.y0)
  shg
  
  figure(h(2))
  subplot(2,1,1)
  plot((1-out.frac)')
  ylabel('PCA err')
  subplot(2,1,2)
  plot(out.v')
  xlabel('# of components')
  ylabel('VARIMAX spars')
  shg
   
  drawnow  
  
  if opt_als.max_iter > 0
    opt_als.mode = 'samples';
    [out.z, out.y] = als(x,out.y0,opt_als);
  else
    out.y = out.y0;
    %out.z = remove_spec_info(x*out.y0);
  end

case 'sparse channels (images)'
  [out.w, out.k_ext, out.frac, out.v, out.z0, out.y] = varimaxfm(x, in.VARIMAX_NumFrac, opt_varimax);

  %out.z0 = remove_spec_info(x*out.w);
  
  figure(h(1));
  showspim(out.z0)
  daspect([1 1 1])
  colormap(jet)
  shg
  
  figure(h(2))
  subplot(2,1,1)
  plot((1-out.frac)')
  ylabel('PCA err')
  subplot(2,1,2)
  plot(out.v')
  xlabel('# of components')
  ylabel('VARIMAX spars')
  shg

  drawnow

  if opt_als.max_iter > 0
    opt_als.mode = 'features';
    [out.z, out.y] = als(x,out.z0,opt_als);
  else
    out.z = out.z0;
    %out.y = dataset(out.w.data.rot\eye(size(out.w.data.rot,1)));
    %out.y.featlab = x.featlab;
    %out.y.featsize = x.featsize;
    %out.y.user = x.user;
  end
end

figure(h(3))
showspim(out.y) 
shg

figure(h(4));
showspim(out.z)
daspect([1 1 1])
colormap(jet)
shg
  
drawnow

tm_putvar(in.Out, out);

return

